/**
 *
 */
package de.renew.extras;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.MenuManager;
import CH.ifa.draw.application.MenuManager.SeparatorFactory;
import CH.ifa.draw.io.exportFormats.ExportFormat;
import CH.ifa.draw.util.CommandMenu;

import de.renew.extras.commands.*;
import de.renew.extras.exports.EPSViewportExportFormat;
import de.renew.extras.exports.IncrementalEPSExportFormat;
import de.renew.extras.exports.TextExportFormat;
import de.renew.extras.layoutwindow.LayoutWindow;
import de.renew.extras.io.importFormats.ImportClCommand;
import de.renew.plugin.PluginAdapter;
import de.renew.plugin.PluginException;
import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JMenuItem;


/**
 * Sandbox for extra functionality that does not belong anywhere (yet).
 *
 * <pre>
 * 0.3 - added lola export feature (experimental)
 *       added stochastic evaluation features
 * </pre>
 *
 * @author Cabac
 *
 */
public class ExtrasPlugin extends PluginAdapter {
    private static final String RENEW_EXTRAS_PLUGIN_NAME = "Renew Extras";
    private static final String INCREMENTAL_EPS_EXPORT = "de.renew.extras.incremental-eps-export";
    private static final String EPS_VIEWPORT_EXPORT = "de.renew.extras.eps-viewport-export";
    private static final String LAYOUT_WINDOW = "de.renew.extras.layoutwindow.enable";
    private static final String LAYOUT_WINDOW_MENU_ITEM = "de.renew.extras.layoutwindow.menuitem";
    private static final String LAYOUT_WINDOW_SEPARATOR = "de.renew.extras.layoutwindow.separator";
    private List<ExportFormat> _formats = new LinkedList<ExportFormat>();
    private CommandMenu _menu;

    /**
     * @param location
     * @throws PluginException
     */
    public ExtrasPlugin(URL location) throws PluginException {
        super(location);
    }

    /**
     * @param props
     */
    public ExtrasPlugin(PluginProperties props) {
        super(props);
    }

    /*
     * (non-Javadoc)
     *
     * @see de.renew.plugin.PluginAdapter#cleanup()
     */
    @Override
    public boolean cleanup() {
        DrawPlugin current = DrawPlugin.getCurrent();
        if (current == null) {
            return true;
        }

        MenuManager mm = current.getMenuManager();

        if (mm != null && _menu != null) {
            mm.unregisterMenu(_menu);
        }
        for (ExportFormat format : _formats) {
            current.getExportHolder().removeExportFormat(format);
        }
        PluginManager pm = PluginManager.getInstance();
        pm.removeCLCommand(SelectionDimensionCommand.CMD);
        pm.removeCLCommand("c");
        pm.removeCLCommand("ls");
        pm.removeCLCommand("pwd");
        pm.removeCLCommand("import");
        pm.removeCLCommand("labels");
        pm.removeCLCommand("renameconcept");
        pm.removeCLCommand(CopyStyleCommand.CMD);
        pm.removeCLCommand(ApplyCopiedStyleCommand.CMD);
        pm.removeCLCommand("reload");

        mm.unregisterMenu(LAYOUT_WINDOW_SEPARATOR);
        mm.unregisterMenu(LAYOUT_WINDOW_MENU_ITEM);
        LayoutWindow.disposeWindow();

        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see de.renew.plugin.PluginAdapter#getAlias()
     */
    @Override
    public String getAlias() {
        return "extras";
    }

    /*
     * (non-Javadoc)
     *
     * @see de.renew.plugin.PluginAdapter#init()
     */
    @Override
    public void init() {
        DrawPlugin current = DrawPlugin.getCurrent();
        if (current == null) {
            return;
        }
        if (PluginProperties.getUserProperties()
                        .getBoolProperty(INCREMENTAL_EPS_EXPORT)) {
            MenuManager mm = current.getMenuManager();
            _menu = new CommandMenu("Extras");
            _menu.add(new GenerateLatexSlides("generate Slide"));
            _menu.putClientProperty(MenuManager.ID_PROPERTY, "de.renew.extras");

            mm.registerMenu(DrawPlugin.TOOLS_MENU, _menu);

            IncrementalEPSExportFormat epsFormat = new IncrementalEPSExportFormat();
            _formats.add(epsFormat);
            current.getExportHolder().addExportFormat(epsFormat);
        }

        if (PluginProperties.getUserProperties()
                        .getBoolProperty(EPS_VIEWPORT_EXPORT)) {
            EPSViewportExportFormat epsViewportFormat = new EPSViewportExportFormat();
            _formats.add(epsViewportFormat);
            current.getExportHolder().addExportFormat(epsViewportFormat);
        }

        TextExportFormat textFormat = new TextExportFormat();
        _formats.add(textFormat);
        current.getExportHolder().addExportFormat(textFormat);

        PluginManager pm = PluginManager.getInstance();
        pm.addCLCommand(SelectionDimensionCommand.CMD,
                        new SelectionDimensionCommand());
        pm.addCLCommand(AttributeCommand.CMD, new AttributeCommand());
        pm.addCLCommand(SimulatorInfoCommand.CMD, new SimulatorInfoCommand());
        pm.addCLCommand(ShellCommand.CMD, new ShellCommand());
        pm.addCLCommand("ls", new ShellCommand("ls"));
        pm.addCLCommand("pwd", new ShellCommand("pwd"));
        pm.addCLCommand("import", new ImportClCommand());
        pm.addCLCommand("labels", new AutoLabelCommand("labels"));
        pm.addCLCommand("renameconcept",
                        new ConceptNameConvertCommand("renameconcept"));
        StyleStore styleStore = new StyleStore();
        pm.addCLCommand(CopyStyleCommand.CMD, new CopyStyleCommand(styleStore));
        pm.addCLCommand(ApplyCopiedStyleCommand.CMD,
                        new ApplyCopiedStyleCommand(styleStore));
        pm.addCLCommand("reload", new ReloadCommand());
        pm.addCLCommand("organize", new ReorganizeDeclarationNodeCommand());

        if (PluginProperties.getUserProperties().getBoolProperty(LAYOUT_WINDOW,
                        true)) {
            addLayoutWindowMenuEntry();
        }
    }

    public static ExtrasPlugin getCurrent() {
        return (ExtrasPlugin) PluginManager.getInstance()
                        .getPluginByName(RENEW_EXTRAS_PLUGIN_NAME);
    }

    private void addLayoutWindowMenuEntry() {
        JMenuItem mi = new JMenuItem("Layout Window");
        mi.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                LayoutWindow.openLayoutWindow();
            }
        });

        SeparatorFactory sepFac = new SeparatorFactory("de.renew.extras");
        JMenuItem separator = sepFac.createSeparator();

        MenuManager mm = DrawPlugin.getCurrent().getMenuManager();
        mm.registerMenu(DrawPlugin.LAYOUT_MENU, separator,
                        LAYOUT_WINDOW_SEPARATOR);
        mm.registerMenu(DrawPlugin.LAYOUT_MENU, mi, LAYOUT_WINDOW_MENU_ITEM);
    }
}