package de.renew.extras.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.figures.AttributeFigure;
import CH.ifa.draw.framework.Figure;

import de.renew.plugin.command.CLCommand;

import java.awt.EventQueue;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;
import java.util.Vector;


/**
 *
 * @author Hewelt
 */
public class AttributeCommand implements CLCommand {
    public final static String CMD = "attr";

    public AttributeCommand() {
    }

    /**
     * Checks the number and type of arguments as well as the selection and
     * displays/manipulates the selected Figure.
     *
     * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[],
     *      java.io.PrintStream)
     */
    @Override
    public void execute(final String[] args, final PrintStream response) {
        try {
            EventQueue.invokeAndWait(new Runnable() {
                @Override
                public void run() {
                    DrawApplication app = DrawPlugin.getGui();
                    Vector<Figure> selection = app.view().selection();

                    if (selection.size() > 1) {
                        response.append("Command " + CMD
                                        + " works only with 1 selected AttributeFigure");
                    } else if (selection.size() == 1) {
                        Figure f = selection.firstElement();
                        if (f instanceof AttributeFigure) {
                            AttributeFigure afig = (AttributeFigure) f;
                            if (args.length == 0) {
                                // display attributes
                                response.append("Figure: " + afig.toString());
                                response.append("\nID: " + afig.getID());
                                response.append("\nAttributes: \n");
                                Enumeration<String> keys = afig
                                                .getAttributeKeys();
                                while (keys.hasMoreElements()) {
                                    String attrName = keys.nextElement();
                                    Object value = afig.getAttribute(attrName);
                                    response.append("  " + attrName + ": "
                                                    + value.toString() + "\n");
                                }
                            } else if (args.length == 1) {
                                // display specified attribute
                                response.append("Figure: " + afig.toString());
                                response.append("\nID: " + afig.getID());
                                Object value = afig.getAttribute(
                                                args[0].toString());
                                if (value != null) {
                                    response.append("\n Attribute: " + args[0]
                                                    + " Value: " + value
                                                    + "\n");
                                } else {
                                    response.append("\nNo attribute " + args[0]
                                                    + "\n");
                                }
                            } else if (args.length == 2) {
                                // set specified attribute
                                String attrName = args[0];
                                String attrValueString = args[1];
                                Object attrValue = null;

                                // TODO Handling of Colors, Lists, Tuples ???
                                if (isNumeric(attrValueString)) {
                                    attrValue = Integer
                                                    .parseInt(attrValueString);
                                } else if (attrValueString.equals("TRUE")
                                                || attrValueString.equals(
                                                                "FALSE")) {
                                    attrValue = Boolean.parseBoolean(
                                                    attrValueString);
                                } else if (attrValueString.equals("NULL")) {
                                    // attrValue is already instantiated to null
                                } else { // treat it as a String
                                    attrValue = attrValueString;
                                    //response.append("Unknown type of value "+attrValueString+". Aborting.\n");
                                    //return;
                                }
                                response.append(afig.hasAttribute(
                                                attrName) ? "Replacing former value " + afig.getAttribute(attrName) + " with new value " + attrValue + "\n" : attrValue == null ? "Removing attribute " + attrName : "Setting attribute " + attrName + " to new value " + attrValue + "\n");
                                afig.setAttribute(attrName, attrValue);
                            } else {
                                response.append(getDescription());
                            }
                        }
                    }
                }
            });
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } //invokeandWait
    }

    /*not used
    static private Rectangle boundingboxof(Vector<Figure> sel) {
        Enumeration<Figure> k = sel.elements();
        Rectangle r = new Rectangle();
        if (k.hasMoreElements()) {
            r = (k.nextElement()).displayBox();
        }
        while (k.hasMoreElements()) {
            r.add((k.nextElement()).displayBox());
        }
        if (r == null) {
            return new Rectangle(0, 0);
        }
        return r;
    }*/
    @Override
    public String getDescription() {
        String descr = "Inspect and set attributes of selected figure\n"
                        + "                Usage:\n"
                        + "		1. attr    -  display all attributes\n"
                        + "		2. attr name   - display attribute <name>\n"
                        + "		3. attr name value - set attribute <name> to <value>\n"
                        + "      	use NULL to remove attribute, TRUE or FALSE to set a boolean value\n"
                        + "		You will need to save the drawing to persist the attributes.";
        return descr;
    }

    /**
     * Simple check, whether a String can be converted into a numeric value. The
     * check uses a simple regexp.
     *
     * @param str
     *            - a given number represented as String
     * @return true if the String is a number
     */
    private static boolean isNumeric(String str) {
        return str.matches("-?\\d+(.\\d+)?");
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getArguments()
     */
    @Override
    public String getArguments() {
        return null;
    }
}