/**
 *
 */
package de.renew.extras.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.ParentFigure;
import CH.ifa.draw.framework.UndoableCommand;

import de.renew.gui.CPNDrawingHelper;
import de.renew.gui.CPNTextFigure;
import de.renew.gui.PlaceFigure;
import de.renew.gui.TransitionFigure;
import de.renew.plugin.command.CLCommand;

import java.io.PrintStream;


/**
 * @author cabac
 *
 */
public class AutoLabelCommand extends UndoableCommand implements CLCommand {
    private String _prefixP = "p";
    private String _prefixT = "t";


    public AutoLabelCommand() {
        this("label");
    }

    /**
     * @param name
     */
    public AutoLabelCommand(String name) {
        super(name);
    }

    /* (non-Javadoc)
     * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[], java.io.PrintStream)
     */
    @Override
    public void execute(String[] args, PrintStream response) {
        if (args.length == 1) {
            _prefixP = args[0] + "p";
            _prefixT = args[0] + "t";
        }
        if (args.length > 1) {
            _prefixP = args[0];
            _prefixT = args[1];
        }
        execute();
    }

    /**
     * @param parent
     * @return
     */
    private boolean hasLabels(ParentFigure parent) {
        FigureEnumeration children = parent.children();
        boolean haslabels = false;
        while (children.hasMoreElements()) {
            Figure child = children.nextElement();
            if (child instanceof CPNTextFigure && ((CPNTextFigure) child)
                            .getType() == CPNTextFigure.LABEL) {
                haslabels = true;
            }
        }
        return haslabels;
    }

    /* (non-Javadoc)
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    @Override
    public String getDescription() {
        return "Will add labels to all nodes, which have no lables yet, in a Petri nets.";
    }

    @Override
    protected boolean executeUndoable() {
        int pp = 0;
        int tt = 0;
        CPNDrawingHelper op = new CPNDrawingHelper();
        DrawApplication gui = DrawPlugin.getGui();
        if (gui != null) {
            Drawing drawing = gui.drawing();
            FigureEnumeration figures = drawing.figures();
            while (figures.hasMoreElements()) {
                Figure figure = figures.nextElement();
                if (figure instanceof PlaceFigure) {
                    PlaceFigure place = (PlaceFigure) figure;
                    if (!hasLabels(place)) {
                        CPNTextFigure textFigure = op.createNameTextFigure(
                                        _prefixP + pp++, place);
                        drawing.add(textFigure);

                    }
                }
                if (figure instanceof TransitionFigure) {
                    TransitionFigure transition = (TransitionFigure) figure;
                    if (!hasLabels(transition)) {
                        CPNTextFigure textFigure = op.createNameTextFigure(
                                        _prefixT + tt++, transition);
                        drawing.add(textFigure);
                    }
                }
            }
            drawing.checkDamage();
        }
        return true;
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getArguments()
     */
    @Override
    public String getArguments() {
        return "drawingNames";
    }
}