package de.renew.extras.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.UndoableCommand;

import de.renew.gui.fs.ConceptFigure;
import de.renew.plugin.command.CLCommand;
import de.renew.util.StringUtil;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;


/**
 * @author Dennis Schmitz
 */
public class ConceptNameConvertCommand extends UndoableCommand
                implements CLCommand {


    public ConceptNameConvertCommand() {
        this("renameconcept");
    }

    /**
     * @param name
     */
    public ConceptNameConvertCommand(String name) {
        super(name);
    }

    /* (non-Javadoc)
     * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[], java.io.PrintStream)
     */
    @Override
    public void execute(String[] args, PrintStream response) {
        execute();
    }

    /* (non-Javadoc)
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    @Override
    public String getDescription() {
        return "Converts all ontology concept names from camel-case (ConceptName) to spinal-case (concept-name).";
    }

    @Override
    protected boolean executeUndoable() {
        DrawApplication gui = DrawPlugin.getGui();
        if (gui != null) {
            List<String> conceptNames = new ArrayList<String>();

            Drawing drawing = gui.drawing();
            FigureEnumeration figures = drawing.figures();

            // collect all concept names
            while (figures.hasMoreElements()) {
                Figure figure = figures.nextElement();
                if (figure instanceof ConceptFigure) {
                    ConceptFigure concept = (ConceptFigure) figure;
                    String conceptName = concept.getName();
                    conceptNames.add(conceptName);
                }
            }

            // sort the concept names
            Collections.sort(conceptNames, new Comparator<String>() {
                @Override
                public int compare(String s1, String s2) {
                    if (s1.length() > s2.length()) {
                        return -1;
                    } else if (s1.length() < s2.length()) {
                        return 1;
                    } else {
                        return 0;
                    }
                }
            });

            // search and replace in all figures
            FigureEnumeration otherFigures = drawing.figures();
            while (otherFigures.hasMoreElements()) {
                Figure otherFigure = otherFigures.nextElement();
                if (otherFigure instanceof ConceptFigure) {
                    ConceptFigure otherConcept = (ConceptFigure) otherFigure;
                    String otherConceptText = otherConcept.getText();

                    String newOtherConceptText = otherConceptText;

                    for (String conceptName : conceptNames) {
                        String newConceptName = StringUtil
                                        .spinalCase(conceptName);
                        // replace all concept names that are followed by a new break
                        newOtherConceptText = newOtherConceptText.replaceAll(
                                        conceptName + "\\n",
                                        newConceptName + "\n");
                        // replace all concept names that are followed by an astrix
                        newOtherConceptText = newOtherConceptText.replaceAll(
                                        conceptName + "\\*",
                                        newConceptName + "*");
                        // replace all concept names that are followed by a line end
                        newOtherConceptText = newOtherConceptText.replaceAll(
                                        conceptName + "$", newConceptName);
                    }
                    otherConcept.setText(newOtherConceptText);
                }
            }
            drawing.checkDamage();
        }
        return true;
    }
}