package de.renew.extras.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.figures.AttributeFigure;
import CH.ifa.draw.framework.Figure;

import de.renew.extras.commands.StyleStore.StyleInformation;
import de.renew.plugin.command.CLCommand;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;


public class CopyStyleCommand implements CLCommand {
    public final static String CMD = "copystyle";
    private StyleStore store;

    public CopyStyleCommand(StyleStore store) {
        this.store = store;
    }

    @Override
    public void execute(final String[] args, final PrintStream response) {
        CommandLineParser parser = new DefaultParser();
        Options opts = new Options();
        opts.addOption("a", "no-attributes", false,
                        "do not copy style attributes");
        opts.addOption("d", "no-dimensions", false, "do not copy dimensions");
        opts.addOption("l", "list", false, "list available style identifiers");
        opts.addOption("h", "help", false, "print help");

        CommandLine line = null;
        try {
            line = parser.parse(opts, args);
        } catch (ParseException e1) {
            response.append("Could not parse command.\n");
            printHelp(response, opts);
            return;
        }

        if (line.hasOption("l")) {
            response.append("Available identifiers: \n");
            for (String identifier : this.store.getKnownIdentifiers()) {
                response.append(identifier + "\n");
            }
            return;
        }

        if (line.hasOption("h")) {
            printHelp(response, opts);
            return;
        }

        List<String> argList = line.getArgList();

        DrawApplication app = DrawPlugin.getGui();
        Vector<Figure> selection = app.view().selection();
        if (selection.size() != 1) {
            response.append("Command " + CMD
                            + ": You need to select exactly one Figure\n");
            printHelp(response, opts);
        } else {
            HashMap<String, Object> attributes = null;
            Dimension dimension = null;
            Figure f = selection.firstElement();

            // copy attributes
            if (!line.hasOption("a")) {
                if (f instanceof AttributeFigure) {
                    attributes = new HashMap<String, Object>();
                    AttributeFigure afig = (AttributeFigure) f;
                    Enumeration<String> attributeKeys = afig.getAttributeKeys();
                    while (attributeKeys.hasMoreElements()) {
                        String key = attributeKeys.nextElement();
                        for (String styleAttribute : StyleStore.STYLE_ATTRIBUTES) {
                            if (styleAttribute.equals(key)) {
                                Object value = afig.getAttribute(key);
                                attributes.put(key, value);
                            }
                        }
                    }
                    response.append("Copied attributes from selected Figure to style store\n");
                } else {
                    response.append("Did not copy attributes from selected Figure to style store, "
                                    + "because the Figure is not an AttributeFigure\n");
                }
            }

            // copy dimensions
            if (!line.hasOption("d")) {
                Rectangle r = f.displayBox();
                dimension = new Dimension(r.width, r.height);
                response.append("Copied dimensions from selected Figure to style store\n");
            }

            StyleInformation style = new StyleStore.StyleInformation(attributes,
                            dimension);
            if (argList.isEmpty()) {
                this.store.storeStyleInformation(style);
                response.append("Store with identifier: "
                                + StyleStore.DEFAULT_NAME + "\n");
            } else {
                for (String identifier : argList) {
                    this.store.storeStyleInformation(identifier, style);
                    response.append("Store with identifier: " + identifier
                                    + "\n");
                }
            }
        }
    }

    @Override
    public String getDescription() {
        return "Copy the style from the selected figure";
    }

    @Override
    public String getArguments() {
        return null;
    }

    private void printHelp(PrintStream response, Options opts) {
        HelpFormatter formatter = new HelpFormatter();
        String header = getDescription();
        formatter.printHelp(new PrintWriter(response, true),
                        HelpFormatter.DEFAULT_WIDTH, CMD + " [identifier]",
                        header, opts, HelpFormatter.DEFAULT_LEFT_PAD,
                        HelpFormatter.DEFAULT_DESC_PAD, null, true);
    }
}