package de.renew.extras.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.contrib.PolygonFigure;
import CH.ifa.draw.figures.AttributeFigure;
import CH.ifa.draw.figures.GroupFigure;
import CH.ifa.draw.figures.PolyLineFigure;
import CH.ifa.draw.figures.RectangleFigure;
import CH.ifa.draw.figures.TextFigure;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.FigureWithID;
import CH.ifa.draw.util.Geom;

import de.renew.extras.ExtrasPlugin;
import de.renew.plugin.command.CLCommand;

import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;


/**
 * Command to display and manipulate (the exact) dimensions of a selected
 * figure.
 *
 * @author cabac
 *
 */
public class SelectionDimensionCommand implements CLCommand {
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(SelectionDimensionCommand.class);
    public final static String CMD = "sd";
    protected static final String ORIGINAL_SIZE_X = "OriginalSizeX";
    protected static final String ORIGINAL_SIZE_Y = "OriginalSizeY";
    protected static final String ORIGINAL_SIZE_W = "OriginalSizeW";
    protected static final String ORIGINAL_SIZE_H = "OriginalSizeH";
    protected static final String ORIGINAL_SIZE_FONT = "OriginalSizeFont";

    public SelectionDimensionCommand() {
    }

    /**
     * Checks the number and type of arguments as well as the selection and
     * displays/manipulates the selected Figure.
     *
     * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[],
     *      java.io.PrintStream)
     */
    @Override
    public void execute(final String[] args, final PrintStream response) {
        try {
            EventQueue.invokeAndWait(new Runnable() {
                @Override
                public void run() {
                    DrawApplication app = DrawPlugin.getGui();
                    app.getUndoRedoManager().prepareUndoSnapshot(app.drawing());
                    ExtrasPlugin plugin = ExtrasPlugin.getCurrent();
                    int scale = plugin.getProperties().getIntProperty(
                                    "de.renew.extras.sd.scale", 1);
                    Vector<Figure> selection = app.view().selection();
                    if (app.view().selectionCount() == 1) {
                        for (int i = 0; i < args.length; i++) {
                            // check all arguments for being numeric
                            if (!isNumeric(args[i])) {
                                response.append(CMD
                                                + " command: All arguments should be numeric.\n"
                                                + getDescription());
                                return;
                            }
                        }
                        Figure f = selection.firstElement();
                        Rectangle r = f.displayBox();
                        boolean changed = false;
                        if (args.length == 0) {
                            // just display dimensions (and Type and id)
                            response.append("Dimensions: ");
                            response.append("x = " + r.x + "      y = " + r.y
                                            + "     w = " + r.width
                                            + "     h = " + r.height + "     "
                                            + "( sd " + r.x + " " + r.y + " "
                                            + r.width + " " + r.height
                                            + " )\n");
                            response.append("Type: "
                                            + f.getClass().getSimpleName()
                                            + "\n");

                            if (f instanceof FigureWithID) {
                                response.append("ID: "
                                                + ((FigureWithID) f).getID()
                                                + "\n");
                            }
                            if (f instanceof RectangleFigure) {
                                response.append("Square: "
                                                + (float) (r.height * r.width)
                                                + "  ( "
                                                + (r.height * r.width) / scale
                                                + " m*m )" + "\n");
                            }
                            if (f instanceof PolyLineFigure) {
                                float length = (float) (Math.sqrt(
                                                r.height ^ 2 + r.width ^ 2));
                                response.append("Length: " + length + "   ( "
                                                + length / scale + " m )"
                                                + "\n");
                            }
                            if (f instanceof PolygonFigure) {
                                PolygonFigure polygon = (PolygonFigure) f;
                                Polygon p = polygon.getPolygon();
                                double area = 0.0;
                                for (int i = 0; i < p.npoints; i++) {
                                    int j = i + 1;
                                    if (j == p.npoints) {
                                        j = 0;
                                    }
                                    area += p.xpoints[i] * p.ypoints[j]
                                                    - p.xpoints[j] * p.ypoints[i];
                                }
                                area /= 2;
                                response.append("Polygon: " + area + "  ( "
                                                + area / scale + " m*m )"
                                                + "\n");
                            }
                        } else if (args.length == 1) {
                            if (args[0].equals("0")) {
                                app.view().clearSelection();
                                app.view().checkDamage();
                            } else {
                                // increase display box by scale factor
                                f.willChange();
                                changed = true;
                                f.displayBox(new Rectangle(r.x, r.y,
                                                (int) (r.width * Float
                                                                .parseFloat(args[0])),
                                                (int) (r.height * Float
                                                                .parseFloat(args[0]))));
                            }
                        } else if (args.length == 2) {
                            // set width and height only
                            f.willChange();
                            changed = true;
                            f.displayBox(new Rectangle(r.x, r.y,
                                            Integer.parseInt(args[0]),
                                            Integer.parseInt(args[1])));

                        } else if (args.length == 3) {
                            //set position and scale by factor (use 1 as factor for move)
                            f.willChange();
                            changed = true;
                            f.displayBox(new Rectangle(
                                            Integer.parseInt(args[0]),
                                            Integer.parseInt(args[1]),
                                            (int) (r.width * Float.parseFloat(
                                                            args[2])),
                                            (int) (r.height * Float.parseFloat(
                                                            args[2]))));
                        } else if (args.length == 4) {
                            // set the rectangle of the selected figure
                            f.willChange();
                            changed = true;
                            f.displayBox(new Rectangle(
                                            Integer.parseInt(args[0]),
                                            Integer.parseInt(args[1]),
                                            Integer.parseInt(args[2]),
                                            Integer.parseInt(args[3])));
                        } else {
                            response.append(getDescription());
                        }
                        if (changed) {
                            app.getUndoRedoManager()
                                            .commitUndoSnapshot(app.drawing());
                            f.changed();
                            app.view().checkDamage();
                        }
                    } else if (app.view().selectionCount() > 1) {
                        // length == 1
                        if (args.length == 1) {
                            Vector<Figure> sel = app.view().selection();

                            // check if we should undo a size change
                            if (args[0].startsWith("u")) {
                                Iterator<Figure> it = sel.iterator();
                                while (it.hasNext()) {
                                    Figure fig = it.next();
                                    if (fig instanceof GroupFigure) {
                                        continue;
                                    }
                                    if (fig instanceof TextFigure) {
                                        TextFigure textfig = (TextFigure) fig;
                                        Font font = textfig.getFont();
                                        Float size = (Float) textfig
                                                        .getAttribute(ORIGINAL_SIZE_FONT);
                                        if (size != null) {
                                            Font oldFont = font
                                                            .deriveFont(size);
                                            textfig.setFont(oldFont);
                                            textfig.setAttribute(
                                                            ORIGINAL_SIZE_FONT,
                                                            null);
                                        }
                                    }
                                    AttributeFigure figure = (AttributeFigure) fig;
                                    if (figure.getAttribute(
                                                    ORIGINAL_SIZE_X) != null) {
                                        Integer x = (Integer) figure
                                                        .getAttribute(ORIGINAL_SIZE_X);
                                        Integer y = (Integer) figure
                                                        .getAttribute(ORIGINAL_SIZE_Y);
                                        Integer w = (Integer) figure
                                                        .getAttribute(ORIGINAL_SIZE_W);
                                        Integer h = (Integer) figure
                                                        .getAttribute(ORIGINAL_SIZE_H);
                                        figure.setAttribute(ORIGINAL_SIZE_X,
                                                        null);
                                        figure.setAttribute(ORIGINAL_SIZE_Y,
                                                        null);
                                        figure.setAttribute(ORIGINAL_SIZE_W,
                                                        null);
                                        figure.setAttribute(ORIGINAL_SIZE_H,
                                                        null);
                                        figure.displayBox(new Rectangle(x, y, w,
                                                        h));
                                        figure.changed();
                                    }
                                } // undo sd
                            } else {
                                float factor = Float.parseFloat(args[0]);
                                Point center = Geom.center(boundingboxof(sel));
                                Iterator<Figure> it = sel.iterator();
                                while (it.hasNext()) {
                                    Figure fig = it.next();
                                    if (fig instanceof GroupFigure) {
                                        continue;
                                    }
                                    if (fig instanceof TextFigure) {
                                        TextFigure textfig = (TextFigure) fig;
                                        Font font = textfig.getFont();
                                        int size = font.getSize();
                                        Font newFont = font.deriveFont(
                                                        size * factor);
                                        textfig.setFont(newFont);
                                        textfig.setAttribute(ORIGINAL_SIZE_FONT,
                                                        size);

                                    }
                                    AttributeFigure figure = (AttributeFigure) fig;
                                    Rectangle r = figure.displayBox();
                                    int dx = center.x - (r.x + r.width / 2);
                                    int dy = center.y - (r.y + r.height / 2);
                                    int dx2 = (int) (dx * factor);
                                    int dy2 = (int) (dy * factor);

                                    int w = (int) (r.width * factor);
                                    int h = (int) (r.height * factor);
                                    int x = r.x + (r.width - w) / 2 + dx - dx2;
                                    int y = r.y + (r.height - h) / 2 + dy - dy2;
                                    if (figure.getAttribute(
                                                    ORIGINAL_SIZE_X) == null) {
                                        figure.setAttribute(ORIGINAL_SIZE_X,
                                                        r.x);
                                        figure.setAttribute(ORIGINAL_SIZE_Y,
                                                        r.y);
                                        figure.setAttribute(ORIGINAL_SIZE_W,
                                                        r.width);
                                        figure.setAttribute(ORIGINAL_SIZE_H,
                                                        r.height);
                                    }
                                    figure.displayBox(
                                                    new Rectangle(x, y, w, h));
                                    figure.changed();
                                }
                            }
                            app.getUndoRedoManager()
                                            .commitUndoSnapshot(app.drawing());
                            app.view().checkDamage();

                        }
                    } else {
                        if (args.length == 0) {
                            response.append("The command '" + CMD
                                            + "' only works on a single selected Figure.\n");
                            FigureEnumeration fe = app.drawing().figures();
                            response.append("Available Figures:\n");
                            while (fe.hasMoreElements()) {
                                Figure fig = fe.nextElement();
                                if (fig instanceof FigureWithID) {
                                    int id = ((FigureWithID) fig).getID();
                                    response.append(fig.getClass()
                                                    .getSimpleName() + "  " + id
                                                    + "\n");
                                }
                            }
                        } else if (args.length == 1) {
                            FigureEnumeration fe = app.drawing().figures();
                            while (fe.hasMoreElements()) {
                                Figure fig = fe.nextElement();
                                if (fig instanceof FigureWithID) {
                                    if (args[0].equals("" + ((FigureWithID) fig)
                                                    .getID())) {
                                        app.view().clearSelection();
                                        app.view().addToSelection(fig);
                                        app.view().checkDamage();
                                        response.append("Figure " + args[0]
                                                        + " selected\n");
                                    }
                                }
                            }
                        }
                    }
                } // run
            } //Runnable
            );
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } //invokeandWait
    }

    static private Rectangle boundingboxof(Vector<Figure> sel) {
        Enumeration<Figure> k = sel.elements();
        Rectangle r = new Rectangle();
        if (k.hasMoreElements()) {
            r = (k.nextElement()).displayBox();
        }
        while (k.hasMoreElements()) {
            r.add((k.nextElement()).displayBox());
        }
        if (r == null) {
            return new Rectangle(0, 0);
        }
        return r;
    }

    @Override
    public String getDescription() {
        String descr = "Usage: " + CMD
                        + " [args]                  (0-4 arguments)\n"
                        + "                0 arguments: " + CMD
                        + "                  (show possible selection ids)\n"
                        + "                1 argument:  " + CMD
                        + " factor           (increase selection by factor)\n"
                        + "                2 arguments: " + CMD
                        + " width height     (set width and height)\n"
                        + "                3 arguments: " + CMD
                        + " x y factor       (set origin (x,y) and increase by factor)\n"
                        + "                4 arguments: " + CMD
                        + " x y width height (set origin, width and height)\n";
        return descr;
    }

    /**
     * Simple check, whether a String can be converted into a numeric value. The
     * check uses a simple regexp.
     *
     * @param str
     *            - a given number represented as String
     * @return true if the String is a number
     */
    private static boolean isNumeric(String str) {
        return str.matches("-?\\d+(.\\d+)?");
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getArguments()
     */
    @Override
    public String getArguments() {
        return null;
    }
}