/**
 *
 */
package de.renew.extras.commands;

import de.renew.plugin.command.CLCommand;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;


/**
 * A command that passes the execution to the shell.
 *
 * @author Lawrence Cabac (cabac@inf...)
 *
 */
public class ShellCommand implements CLCommand {
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(ShellCommand.class);
    public final static String CMD = "!";

    /**
    *
    */
    private String commandname;
    private boolean customShellCommand = false;

    /**
     * Customize the shell command by giving command name. Convenient as it is,
     * the command is OS dependent.
     *
     * @param commandname
     *            - name of shell commmand
     */
    public ShellCommand(String commandname) {
        super();
        this.commandname = commandname;
        customShellCommand = true;
    }

    /**
     * Constructor for the standard variant of the command. Renew command is
     * "c". The first argument will be executed in the shell.
     */
    public ShellCommand() {
        commandname = "";
    }

    /**
     * A execute passes the command to the Runtime for execution. If a
     * customized command name is given this command is executed, otherwise the
     * first argument to the "c" command is executed as command.
     *
     * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[],
     *      java.io.PrintStream)
     */
    @Override
    public void execute(String[] args, PrintStream response) {
        String arguments = " ";
        if (!customShellCommand && args.length == 0) {
            response.append("Please provide an appropriate command line command,\n"
                            + "that can be executed under "
                            + System.getProperty("os.name") + ".\n");
            return;
        }
        for (String string : args) {
            arguments += " " + string;
        }
        try {
            Process exec = Runtime.getRuntime().exec(commandname + arguments);
            InputStream is = exec.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);

            PrintWriter writer = new PrintWriter(response, true);
            BufferedWriter bw = new BufferedWriter(writer);

            String read = "";
            while ((read = br.readLine()) != null) {
                bw.write(read + "\n");
            }

            br.close();
            isr.close();
            is.close();

            bw.flush();
            // writer.close();
            // bw.close();
        } catch (IOException e) {
            if (logger.isDebugEnabled()) {
                logger.debug(ShellCommand.class.getName() + ": ", e);
            }
            response.append(e.getMessage() + "\n");
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    @Override
    public String getDescription() {
        return "Executes a command in the OS's shell: " + commandname;
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getArguments()
     */
    @Override
    public String getArguments() {
        if (commandname.equals("ls")) {
            return "[fileNames]";
        }
        return null;
    }
}