package de.renew.extras.commands;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingView;

import de.renew.gui.CPNInstanceDrawing;
import de.renew.net.NetElementID;
import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginManager;
import de.renew.plugin.command.CLCommand;
import de.renew.remote.BindingAccessor;
import de.renew.remote.NetInstanceAccessor;
import de.renew.remote.TransitionInstanceAccessor;
import de.renew.remote.TriggerableForwarder;

import java.awt.EventQueue;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;


/**
 *
 * @author Hewelt
 */
public class SimulatorInfoCommand implements CLCommand {
    public final static String CMD = "sim";

    public SimulatorInfoCommand() {
    }

    /**
     * Checks the number and type of arguments as well as the selection and
     * displays/manipulates the selected Figure.
     *
     * @see de.renew.plugin.command.CLCommand#execute(java.lang.String[],
     *      java.io.PrintStream)
     */
    @Override
    public void execute(final String[] args, final PrintStream response) {
        try {
            EventQueue.invokeAndWait(new Runnable() {
                @Override
                public void run() {
                    DrawApplication app = DrawPlugin.getGui();
                    DrawingView view = app.view();
                    Drawing draw = view.drawing();
                    response.append("=====================================================================\n");
                    response.append("Simulation Information Command: \n");
                    response.append("=====================================================================\n");
                    if (draw instanceof CPNInstanceDrawing) {
                        response.append("Drawing is a CPNInstanceDrawing: "
                                        + prepareTextForConsole(draw.getName(),
                                                        "", draw.getName())
                                        + "\n");
                        CPNInstanceDrawing inst = (CPNInstanceDrawing) draw;
                        NetInstanceAccessor net = inst.getNetInstance();
                        List<TransitionInstanceAccessor> trans = new LinkedList<TransitionInstanceAccessor>();
                        try {
                            NetElementID[] transIds = net.getNet()
                                            .getTransitionIDs();
                            for (NetElementID id : transIds) {
                                TransitionInstanceAccessor tr = net
                                                .getTransitionInstance(id);
                                trans.add(tr);
                                BindingAccessor[] allBindings = tr
                                                .findAllBindings(
                                                                new TriggerableForwarder() {
                                                                    @Override
                                                                    public void proposeSearch()
                                                                                    throws RemoteException {
                                                                        // TODO Auto-generated method stub
                                                                    }
                                                                });
                                if (allBindings.length > 0) {
                                    response.append("-----------------------------------------------------------\n");
                                    response.append(prepareTextForConsole(tr
                                                    .asString() + ": "
                                                    + allBindings.length
                                                    + "  Bindings\n", " ",
                                                    draw.getName()));
                                }
                                for (BindingAccessor ba : allBindings) {
                                    response.append("  "
                                                    + prepareTextForConsole(ba
                                                                    .getDescription(),
                                                                    "  ",
                                                                    draw.getName())
                                                    + "\n");
                                }
                            }

                            //TransitionInstanceAccessor[] trans = l.toArray(new TransitionInstanceAccessor[0]);
                        } catch (RemoteException e) {
                            // TODO Auto-generated catch block
                            e.printStackTrace();
                        }
                    } else {
                        response.append("No instance drawing in focus. Did you start a simulation?\n");
                    }
                }
            });
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } //invokeandWait
    }

    /*not used
    static private Rectangle boundingboxof(Vector<Figure> sel) {
        Enumeration<Figure> k = sel.elements();
        Rectangle r = new Rectangle();
        if (k.hasMoreElements()) {
            r = (k.nextElement()).displayBox();
        }
        while (k.hasMoreElements()) {
            r.add((k.nextElement()).displayBox());
        }
        if (r == null) {
            return new Rectangle(0, 0);
        }
        return r;
    }*/
    @Override
    public String getDescription() {
        String descr = "Provides information about the current simulation state.";
        return descr;
    }

    private String prepareTextForConsole(String description, String indentation,
                                         String drawingName) {
        IPlugin plugin = PluginManager.getInstance().getPluginByName("Console");
        if (plugin != null) {
            // 31 red | 34 blue | 32 green | 33 broken yellow
            if (plugin.getProperties()
                            .getBoolProperty("de.renew.console.color")) {
                StringBuffer result = new StringBuffer();

                for (String line : description.split("\n")) {
                    // colorize variable before =
                    String line0 = line.replaceAll("([\\w]+)=",
                                    "\u001B[31m$1\u001B[0m=");

                    // colorize transition name in binding
                    String line1 = line0.replaceAll("\\.(.+)\\{",
                                    ".\u001B[34m$1\u001B[0m{");

                    // colorize transition name in heading
                    String line2 = line1.replaceAll("\\.(.+):",
                                    ".\u001B[34m$1\u001B[0m:");

                    // colorize instance drawing name
                    String line3 = line2.replace(drawingName,
                                    "\u001B[33m" + drawingName + "\u001B[0m");
                    result.append(indentation + line3 + "\n");
                }
                return result.toString();
            } // color set to false
        } // no console plugin available
        return description;
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getArguments()
     */
    @Override
    public String getArguments() {
        return null;
    }

    /*
    /**
     * Simple check, whether a String can be converted into a numeric value. The
     * check uses a simple regexp.
     *
     * @param str
     *            - a given number represented as String
     * @return true if the String is a number
     */


    /*
    private static boolean isNumeric(String str) {
     return str.matches("-?\\d+(.\\d+)?");
    }*/
}