package de.renew.extras.commands;

import CH.ifa.draw.figures.TextFigure;

import java.awt.Color;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;


public class StyleStore {
    public final static String[] STYLE_ATTRIBUTES = { "FrameColor", "FillColor", "TextColor", TextFigure.ALIGN_ATTR, "ArrowMode", "FontName", "FontSize", "FontStyle", "LineShape", "BSplineSegments", "BSplineDegree" };
    public final static String DEFAULT_NAME = "default";
    private Map<String, StyleInformation> storedStyles;

    public StyleStore() {
        this.storedStyles = new HashMap<String, StyleInformation>();
        saveStylePresets();
    }

    private void saveStylePresets() {
        HashMap<String, Object> attributes;
        StyleInformation style;

        // manual
        attributes = new HashMap<String, Object>();
        attributes.put("FrameColor", Color.red);
        attributes.put("FillColor", Color.orange);
        style = new StyleInformation(attributes, null);
        storeStyleInformation("manual", style);

        // ncbg (NetComponent background)
        attributes = new HashMap<String, Object>();
        attributes.put("FrameColor", new Color(224, 224, 224));
        attributes.put("FillColor", new Color(240, 240, 240));
        style = new StyleInformation(attributes, null);
        storeStyleInformation("ncbg", style);
    }

    public void storeStyleInformation(String identifier,
                                      StyleInformation info) {
        this.storedStyles.put(identifier, info);
    }

    public void storeStyleInformation(StyleInformation info) {
        this.storedStyles.put(DEFAULT_NAME, info);
    }

    public StyleInformation getStyleInformation(String identifier) {
        return this.storedStyles.get(identifier);
    }

    public StyleInformation getStyleInformation() {
        return this.storedStyles.get(DEFAULT_NAME);
    }

    public Set<String> getKnownIdentifiers() {
        return this.storedStyles.keySet();
    }

    public static class StyleInformation {
        private Map<String, Object> attributes;
        private Dimension dimensions;

        public StyleInformation(Map<String, Object> attributes,
                                Dimension dimensions) {
            this.attributes = attributes;
            this.dimensions = dimensions;
        }

        public Map<String, Object> getAttributes() {
            return this.attributes;
        }

        public Dimension getDimensions() {
            return this.dimensions;
        }
    }
}