package de.renew.extras.exports;

import de.renew.util.StringUtil;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;


public class CreateLatexSlide {
    //LaTeX slide styles known
    // if you add more notice implicit decisions in the code
    public static final int PROSPER = 0;
    public static final int BEAMER = 1;

    //default LaTeX style is prosper
    private static int targetStyle = PROSPER;

    //default is to write main LaTeX file with import
    private static boolean writeMain = true;
    private static String _path = "./";

    private static String getSlidePrefix(String slideName, int overlays) {
        if (targetStyle == PROSPER) {
            return "\\overlays{" + overlays + "}{\n\\begin{slide}{" + slideName
                            + "}\n\n";
        } else {
            return "\n\\begin{frame}\n\\frametitle{" + slideName + "}\n\n";
        }
    }

    private static String getSlideSuffix() {
        if (targetStyle == PROSPER) {
            return "\\end{slide}\n}\n\n";
        } else {
            return "\\end{frame}\n";
        }
    }

    private static File[] getFilesByPrefix(String filePrefix) {
        File absDir = new File(System.getProperty("user.dir"));
        return absDir.listFiles(new PrefixEPSFileFilter(filePrefix));
    }

    private static File[] getFilesByPrefix(String path, String filePrefix) {
        File absDir = new File(path);
        return absDir.listFiles(new PrefixEPSFileFilter(filePrefix));
    }

    private static void writeMainFile(String path, String filePrefix) {
        try {
            String preambleEscapedTitles = "\\usepackage{fancyvrb}\n"
                            + "\\DefineShortVerb{\\|}\n"
                            + "\\SaveVerb{Title}|testSlides_1|\n";
            String escapedFilePrefix = "\\protect\\UseVerb{Title}";

            FileWriter fout = new FileWriter(
                            path + File.separator + filePrefix + "_main.tex");
            if (targetStyle == PROSPER) {
                fout.write("\\documentclass[pdf,myazure]{prosper}\n"
                                + preambleEscapedTitles
                                + "\\usepackage{graphics}\n\\begin{document}\n\\input{"
                                + path + File.separator + filePrefix
                                + ".tex}\n\\end{document}\n");
            } else { //beamer
                fout.write("\\documentclass{beamer}\n" + preambleEscapedTitles
                                + "\\usepackage{graphics}\n\\mode<presentation>\n"
                                + "\\title{" + escapedFilePrefix
                                + "}\n\\begin{document}\n\\begin{frame}\n\\titlepage\n\\end{frame}\n\n"
                                + "\\input{" + path + File.separator
                                + filePrefix + ".tex}\n\\end{document}\n");
            }
            fout.close();
        } catch (IOException e) {
            e.printStackTrace();
            //application.showStatus("Export to LaTeX slide show failed");
            System.out.println("Export to LaTeX slide show failed.");
        }
    }

    public static void writeLaTeXSlide(String pathPrefix, int target) {
        targetStyle = target;
        String path = StringUtil.getPath(pathPrefix);
        String pre = StringUtil.getFilename(pathPrefix);
        if (path != null && !path.equals("")) {
            _path = path;
        }
        writeLaTeXSlide(pre);
    }

    public static void writeLaTeXSlide(String filePrefix) {
        File[] files = null;
        if (_path == "./") {
            files = getFilesByPrefix(filePrefix);
        } else {
            files = getFilesByPrefix(_path, filePrefix);
        }
        String escapedFilePrefix = "\\protect\\UseVerb{Title}";

        if (files.length == 0) {
            System.out.println("Given file prefix: " + _path + File.separator
                            + filePrefix
                            + " does not match any file in current directory.");
        } else {
            //construct slide
            StringBuffer slide = new StringBuffer(
                            getSlidePrefix(escapedFilePrefix, files.length));

            Arrays.sort(files);
            for (int i = 0; i < files.length; i++) {
                if (targetStyle == PROSPER) {
                    slide.append("\\onlySlide*{" + (i + 1) + "}{\\centerline{");
                    slide.append("\\includegraphics[width=\\textwidth]{");
                    slide.append(files[i].getName() + "}}}\n");
                } else {
                    slide.append("\\includegraphics<" + (i + 1)
                                    + ">[width=\\textwidth]{");
                    slide.append(files[i].getName() + "}\n");
                }
            }
            slide.append(getSlideSuffix());

            //save slide
            try {
                FileWriter fout = new FileWriter(
                                _path + File.separator + filePrefix + ".tex");
                fout.write(slide.toString());
                fout.close();
            } catch (IOException e) {
                e.printStackTrace();
                //application.showStatus("Export to LaTeX slide failed");
                System.out.println("Export to LaTeX slide failed.");
            }

            //generate main LaTeX file, if wanted
            if (writeMain) {
                writeMainFile(_path, filePrefix);
            }
        }
    }


    /**
     * @param args
     *
     * Usage: java -cp extras.jar de.renew.extras.exports.CreateLatexSlide [file prefix] [LaTeX style]
     * e.g.: java -cp eclipse-Workspace/Renew/dist/plugins/extras.jar de.renew.extras.exports.CreateLatexSlide test1-slide prosper
     */
    public static void main(String[] args) {
        String filePrefix = new String();

        //LaTeX Style
        if (args.length > 1) {
            try {
                if (args[1].equals("beamer")) {
                    targetStyle = BEAMER;
                }
                System.out.println("Using LaTeX Style: " + targetStyle);
            } catch (ArrayIndexOutOfBoundsException ae) {
            }

            //this should not happen... 
        } else {
            System.out.println("Using default LaTeX Style: prosper");
        }

        //File Prefix
        try {
            filePrefix = args[0];
        } catch (ArrayIndexOutOfBoundsException ae) {
            System.out.println(
                            "\nUsage: java CreateLatexSlide <FilenamePrefix> [prosper|beamer]");
        }

        writeLaTeXSlide(filePrefix, targetStyle);
    }
}