package de.renew.extras.exports;

import CH.ifa.draw.figures.AttributeFigure;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.io.EPSFileFilter;

import de.renew.export.io.exportFormats.EPSExportFormat;
import de.renew.util.StringUtil;

import java.awt.Rectangle;
import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;


public class EPSViewportExportFormat extends EPSExportFormat {
    private static final String VIEWPORT_ATTRIBUTE = "viewport";
    private static final String EXPORT_FILE_NAME_PREFIX = "_viewport_";

    public EPSViewportExportFormat() {
        super();
        setFormatName("EPS-Viewport");
    }

    @Override
    public File export(Drawing drawing, File file) throws Exception {
        File result = null;
        Map<Figure, Boolean> viewportVisibility = new HashMap<Figure, Boolean>();
        try {
            TreeMap<Integer, Rectangle> viewports = new TreeMap<Integer, Rectangle>();

            FigureEnumeration figures = drawing.figures();
            while (figures.hasMoreElements()) {
                Figure fig = figures.nextFigure();
                if (fig instanceof AttributeFigure) {
                    AttributeFigure attrfig = (AttributeFigure) fig;
                    Object attribute = attrfig.getAttribute(VIEWPORT_ATTRIBUTE);
                    if (attribute != null) {
                        Rectangle bounds = attrfig.displayBox();
                        viewportVisibility.put(attrfig, attrfig.isVisible());
                        attrfig.setVisible(false);
                        Rectangle viewport = null;
                        FigureEnumeration figures2 = drawing.figures();
                        while (figures2.hasMoreElements()) {
                            Figure fig2 = figures2.nextFigure();
                            Rectangle r = fig2.displayBox();
                            if (fig2 != fig && bounds.contains(r)) {
                                if (viewport == null) {
                                    viewport = r;
                                } else {
                                    viewport.add(r);
                                }
                            }
                        }
                        viewport.grow(10, 10);
                        if (attribute instanceof Integer) {
                            viewports.put((Integer) attribute, viewport);
                        }
                    }
                }
            }

            String stripedName = StringUtil
                            .stripFilenameExtension(file.getAbsolutePath());

            // int number = 1;
            for (Entry<Integer, Rectangle> entry : viewports.entrySet()) {
                int number = entry.getKey();
                Rectangle viewport = entry.getValue();
                String viewportPath = stripedName + EXPORT_FILE_NAME_PREFIX
                                + number + "."
                                + new EPSFileFilter().getExtension();
                File viewportFile = new File(viewportPath);

                internalExport(drawing, viewportFile, viewport, true);
                // number++;
            }

            Rectangle bounds = drawing.getBounds();
            result = internalExport(drawing, file, bounds, true);
        } finally {
            for (Entry<Figure, Boolean> entry : viewportVisibility.entrySet()) {
                entry.getKey().setVisible(entry.getValue());
            }
        }

        return result;
    }

    @Override
    public int getShortCut() {
        return -1;
    }
}