package de.renew.extras.exports;

import org.freehep.graphics2d.VectorGraphics;
import org.freehep.graphicsbase.util.UserProperties;
import org.freehep.graphicsio.PageConstants;
import org.freehep.graphicsio.ps.AbstractPSGraphics2D;
import org.freehep.graphicsio.ps.PSGraphics2D;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.io.EPSFileFilter;
import CH.ifa.draw.io.exportFormats.ExportFormatAbstract;

import de.renew.util.StringUtil;

import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.JPanel;

import org.freehep.graphics2d.VectorGraphics;
import org.freehep.graphicsbase.util.UserProperties;
import org.freehep.graphicsio.PageConstants;
import org.freehep.graphicsio.ps.PSGraphics2D;


public class IncrementalEPSExportFormat extends ExportFormatAbstract {
    private static String exportFileNamePrefix;

    // Attributes
    // Construktor
    public IncrementalEPSExportFormat() {
        super("EPS++", new EPSFileFilter());
    }

    // Methods


    /**
     * {@link de.renew.export.io.exportFormats.EPSExportFormat#canExportNto1()}
     */
    @Override
    public boolean canExportNto1() {
        return false;
    }

    /**
     * {@link de.renew.export.io.exportFormats.EPSExportFormat#export(CH.ifa.draw.framework.Drawing, java.io.File)}
     */
    @Override
    public File export(Drawing drawing, File path) throws Exception {
        File result = null;
        if (drawing != null && path != null) {
            String stripedName = StringUtil
                            .stripFilenameExtension(path.getAbsolutePath());
            System.out.println("IncrementalEPSExportFormat" + stripedName);
            stripedName = stripedName.replace('[', '_');
            stripedName = stripedName.replace(']', '_');

            String exportFileName = "";
            int i = 0;
            do {
                String number = "" + (1000 + i++);
                number = number.substring(1, number.length());
                System.out.println("number : " + number);
                exportFileNamePrefix = stripedName + "_slide_";
                exportFileName = exportFileNamePrefix + number + "."
                                + new EPSFileFilter().getExtension();
                result = new File(exportFileName);
            } while (result.exists());

            JPanel drawingPanel = (JPanel) DrawPlugin.getGui().getView(drawing);

            VectorGraphics graphics = new PSGraphics2D(result, drawingPanel);

            final UserProperties defaultProperties = new UserProperties();
            defaultProperties.setProperty(AbstractPSGraphics2D.PAGE_SIZE,
                            PageConstants.A4);
            graphics.setProperties(defaultProperties);

            //Start exporting image to eps
            graphics.startExport();

            //Paint the picture
            drawingPanel.print(graphics);

            //End exporting
            graphics.endExport();
        }
        assert (result != null) : "Failure in EPSExportFormat: result == null";
        return result;
    }

    /**
     * {@link de.renew.export.io.exportFormats.EPSExportFormat#export(CH.ifa.draw.framework.Drawing[], java.io.File)}
     */
    @Override
    public File export(Drawing[] drawings, File path) throws Exception {
        File result = null;
        assert (result != null) : "Failure in EPSExportFormat: result == null";
        return result;
    }

    @Override
    public boolean canExportDrawing(Drawing drawing) {
        return true;
    }

    @Override
    public int getShortCut() {
        return KeyEvent.VK_E;
    }

    /* (non-Javadoc)
     * modifier is the standard for Renew Ctrl. resp. Command plus the shift key.
     * @see CH.ifa.draw.io.exportFormats.ExportFormatAbstract#getModifier()
     */
    @Override
    public int getModifier() {
        return Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()
                        + InputEvent.SHIFT_DOWN_MASK;
    }

    /* (non-Javadoc)
     * @see CH.ifa.draw.io.exportFormats.ExportFormatAbstract#forceGivenName()
     */
    @Override
    public boolean forceGivenName() {
        return true;
    }

    /**
     * @return Returns the exportFileNamePrefix.
     */
    public static String getExportFileNamePrefix() {
        return exportFileNamePrefix;
    }

    /**
     * @param exportFileNamePre The exportFileNamePrefix to set.
     */
    public static void setExportFileNamePrefix(String exportFileNamePre) {
        exportFileNamePrefix = exportFileNamePre;
    }
}