package de.renew.extras.exports;

import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.FigureWithID;
import CH.ifa.draw.framework.ParentFigure;
import CH.ifa.draw.io.SimpleFileFilter;
import CH.ifa.draw.io.exportFormats.ExportFormatAbstract;

import de.renew.gui.ArcConnection;
import de.renew.gui.CPNDrawing;
import de.renew.gui.CPNTextFigure;
import de.renew.gui.PlaceFigure;
import de.renew.gui.TransitionFigure;
import de.renew.util.StringUtil;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.util.List;


public class TextExportFormat extends ExportFormatAbstract {
    public TextExportFormat() {
        super("Text", new SimpleFileFilter("txt", "Text file"));
    }

    @Override
    public File export(Drawing drawing, File path) throws Exception {
        File result = null;
        if (drawing != null && path != null) {
            result = path;
            FileOutputStream stream = new FileOutputStream(result);
            PrintWriter printWriter = new PrintWriter(
                            new OutputStreamWriter(stream, "UTF-8"));
            FigureEnumeration figures = drawing.figures();
            List<String> places = new LinkedList<String>();
            List<String> transitions = new LinkedList<String>();
            List<String> arcs = new LinkedList<String>();

            while (figures.hasMoreElements()) {
                Figure fig = figures.nextElement();
                if (fig instanceof PlaceFigure) {
                    PlaceFigure pf = (PlaceFigure) fig;
                    String name = getNodeName(pf);
                    places.add(name);
                } else if (fig instanceof TransitionFigure) {
                    TransitionFigure tf = (TransitionFigure) fig;
                    String name = getNodeName(tf);
                    transitions.add(name);
                } else if (fig instanceof ArcConnection) {
                    ArcConnection ac = (ArcConnection) fig;
                    Figure start = ac.startFigure();
                    String startName = getNodeName(start);
                    Figure end = ac.endFigure();
                    String endName = getNodeName(end);
                    arcs.add("(" + startName + ", " + endName + ")");
                }
            }
            printWriter.write("N = (P, T, F)\n");
            printWriter.write("P = {" + StringUtil.join(places, ", ") + "}\n");
            printWriter.write("T = {" + StringUtil.join(transitions, ", ")
                            + "}\n");
            printWriter.write("F = {" + StringUtil.join(arcs, ", ") + "}\n");

            printWriter.close();
        }
        return result;
    }

    private String getNodeName(Figure figure) {
        String name = null;
        if (figure instanceof ParentFigure) {
            FigureEnumeration children = ((ParentFigure) figure).children();

            while (children.hasMoreElements()) {
                Figure child = children.nextElement();
                if (child instanceof CPNTextFigure) {
                    CPNTextFigure text = (CPNTextFigure) child;
                    if (text.getType() == CPNTextFigure.NAME) {
                        name = text.getText();
                        break;
                    }
                }
            }
        }
        if (name == null) {
            if (figure instanceof PlaceFigure) {
                name = "p" + ((FigureWithID) figure).getID();
            } else if (figure instanceof TransitionFigure) {
                name = "t" + ((FigureWithID) figure).getID();
            } else if (figure instanceof ArcConnection) {
                name = "a" + ((FigureWithID) figure).getID();
            }
        }
        return name;
    }

    @Override
    public File export(Drawing[] drawings, File path) throws Exception {
        return null;
    }

    @Override
    public boolean canExportNto1() {
        return false;
    }

    @Override
    public boolean canExportDrawing(Drawing drawing) {
        boolean result = false;
        if (drawing instanceof CPNDrawing) {
            result = true;
        }
        return result;
    }
}