package de.renew.extras.layoutwindow;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.figures.GroupCommand;
import CH.ifa.draw.figures.UngroupCommand;
import CH.ifa.draw.standard.AlignCommand;
import CH.ifa.draw.standard.SpreadCommand;
import CH.ifa.draw.util.Iconkit;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.NumberFormat;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.text.NumberFormatter;


/**
 * This window can be used to align, spread, group and ungroup figures. It
 * basically provides a window as alternative to the menu commands, but adds a
 * simple feature for aligning figures. One can select the anchor figure for the
 * alignment. One can chose between "Last selected", "First selected",
 * "Biggest object", "Smallest object", "Selection". The group/ungroup commands
 * are the default commands from the "Edit"-menu and not the NetComponents
 * commands. This window shall help with the positioning of many figures.
 *
 * @author 6hauster
 */
public class LayoutWindow extends JFrame {
    private final static String IMAGES = "Extras/src/main/resources/de/renew/extras/images/";
    private static LayoutWindow layoutWindow;
    private static PositionDimensionsTracker positionDimensionsTracker;

    public LayoutWindow() {
        super("Layout");
    }

    public void init() {
        Container contentPane = getContentPane();
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
        createAlignPanel(contentPane);
        createDistributePanel(contentPane);
        createPositionDimensionsPanel(contentPane);
        createGroupUngroup(contentPane);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                DrawPlugin.getCurrent().getMenuManager().getWindowsMenu()
                                .removeFrame(LayoutWindow.this);
            }
        });
    }

    private void createAlignPanel(Container contentPane) {
        JPanel alignPanel = new JPanel(new BorderLayout());
        TitledBorder alignBorder = BorderFactory.createTitledBorder("Align");
        alignPanel.setBorder(alignBorder);

        JPanel relativePanel = new JPanel();
        relativePanel.setLayout(
                        new BoxLayout(relativePanel, BoxLayout.LINE_AXIS));
        relativePanel.add(new JLabel("Relative to:"));

        String[] listElements = new String[] { "Last selected", "First selected", "Biggest object", "Smallest object", "Selection" };
        JComboBox<String> jComboBox = new JComboBox<String>(listElements);
        relativePanel.add(jComboBox);

        alignPanel.add(relativePanel, BorderLayout.NORTH);
        JPanel alignPanelButtons = new JPanel(new GridLayout(2, 5));
        alignPanel.add(alignPanelButtons, BorderLayout.CENTER);

        String iconPath;
        ImageIcon icon;
        JButton button;

        iconPath = IMAGES + "align/lefts.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Lefts", icon);
        button.addActionListener(new AlignCommandActionListener("LEFTS",
                        AlignCommand.LEFTS, jComboBox));
        alignPanelButtons.add(button);

        iconPath = IMAGES + "align/centers.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Centers", icon);
        button.addActionListener(new AlignCommandActionListener("CENTERS",
                        AlignCommand.CENTERS, jComboBox));
        alignPanelButtons.add(button);

        iconPath = IMAGES + "align/rights.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Rights", icon);
        button.addActionListener(new AlignCommandActionListener("RIGHTS",
                        AlignCommand.RIGHTS, jComboBox));
        alignPanelButtons.add(button);

        iconPath = IMAGES + "align/tops.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Tops", icon);
        button.addActionListener(new AlignCommandActionListener("TOPS",
                        AlignCommand.TOPS, jComboBox));
        alignPanelButtons.add(button);

        iconPath = IMAGES + "align/middles.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Middles", icon);
        button.addActionListener(new AlignCommandActionListener("MIDDLES",
                        AlignCommand.MIDDLES, jComboBox));
        alignPanelButtons.add(button);

        iconPath = IMAGES + "align/bottoms.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Bottoms", icon);
        button.addActionListener(new AlignCommandActionListener("BOTTOMS",
                        AlignCommand.BOTTOMS, jComboBox));
        alignPanelButtons.add(button);

        contentPane.add(alignPanel);
    }

    private void createDistributePanel(Container contentPane) {
        JPanel distributePanel = new JPanel();
        distributePanel.setLayout(
                        new BoxLayout(distributePanel, BoxLayout.PAGE_AXIS));
        TitledBorder distributeBorder = BorderFactory
                        .createTitledBorder("Spread");
        distributePanel.setBorder(distributeBorder);

        JPanel horizontal = new JPanel(new GridLayout(0, 3));
        distributePanel.add(horizontal);
        JPanel vertical = new JPanel(new GridLayout(0, 3));
        distributePanel.add(vertical);
        JPanel diagonal = new JPanel(new GridLayout(0, 3));
        distributePanel.add(diagonal);

        String iconPath;
        ImageIcon icon;
        JButton button;

        // horizontal
        iconPath = IMAGES + "spread/lefts.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Lefts", icon);
        button.addActionListener(new CommandActionListener(
                        new SpreadCommand("LEFTS", SpreadCommand.LEFTS)));
        horizontal.add(button);

        iconPath = IMAGES + "spread/centers.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Centers", icon);
        button.addActionListener(new CommandActionListener(
                        new SpreadCommand("CENTERS", SpreadCommand.CENTERS)));
        horizontal.add(button);

        iconPath = IMAGES + "spread/rights.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Rights", icon);
        button.addActionListener(new CommandActionListener(
                        new SpreadCommand("RIGHTS", SpreadCommand.RIGHTS)));
        horizontal.add(button);

        iconPath = IMAGES + "spread/distances-lr.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Distances", icon);
        button.addActionListener(new CommandActionListener(new SpreadCommand(
                        "DISTANCES", SpreadCommand.HORIZONTAL_DISTANCE)));
        horizontal.add(button);

        // vertical
        iconPath = IMAGES + "spread/tops.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Tops", icon);
        button.addActionListener(new CommandActionListener(
                        new SpreadCommand("TOPS", SpreadCommand.TOPS)));
        vertical.add(button);

        iconPath = IMAGES + "spread/middles.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Middles", icon);
        button.addActionListener(new CommandActionListener(
                        new SpreadCommand("MIDDLES", SpreadCommand.MIDDLES)));
        vertical.add(button);

        iconPath = IMAGES + "spread/bottoms.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Bottoms", icon);
        button.addActionListener(new CommandActionListener(
                        new SpreadCommand("BOTTOMS", SpreadCommand.BOTTOMS)));
        vertical.add(button);

        iconPath = IMAGES + "spread/distances-tb.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Distances", icon);
        button.addActionListener(new CommandActionListener(new SpreadCommand(
                        "DISTANCES", SpreadCommand.VERTICAL_DISTANCE)));
        vertical.add(button);

        // diagonal
        iconPath = IMAGES + "spread/diagonal.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Diagonal", icon);
        button.addActionListener(new CommandActionListener(new SpreadCommand(
                        "DIAGONAL", SpreadCommand.DIAGONAL_CENTERS)));
        diagonal.add(button);

        contentPane.add(distributePanel);
    }
    
    private void createPositionDimensionsPanel(Container contentPane) {
        JPanel positionDimensionsPanel = new JPanel();
        positionDimensionsPanel.setLayout(new BoxLayout(positionDimensionsPanel, BoxLayout.X_AXIS));
        TitledBorder groupUngroupBorder = BorderFactory
                        .createTitledBorder("Position/Dimensions");
        positionDimensionsPanel.setBorder(groupUngroupBorder);
        
        NumberFormat intFormat = NumberFormat.getIntegerInstance();
        intFormat.setGroupingUsed(false);
        NumberFormatter numberFormatter = new NumberFormatter(intFormat);
        
        JLabel xLabel = new JLabel("x", SwingConstants.RIGHT);
        JFormattedTextField xTextField = new JFormattedTextField(numberFormatter);

        positionDimensionsPanel.add(xLabel);
        positionDimensionsPanel.add(xTextField);
        
        JLabel yLabel = new JLabel("y", SwingConstants.RIGHT);
        JFormattedTextField yTextField = new JFormattedTextField(numberFormatter);
        positionDimensionsPanel.add(yLabel);
        positionDimensionsPanel.add(yTextField);
        
        JLabel widthLabel = new JLabel("width", SwingConstants.RIGHT);
        JFormattedTextField widthTextField = new JFormattedTextField(numberFormatter);
        positionDimensionsPanel.add(widthLabel);
        positionDimensionsPanel.add(widthTextField);
        
        JLabel heightLabel = new JLabel("height", SwingConstants.RIGHT);
        JFormattedTextField heightTextField = new JFormattedTextField(numberFormatter);
        positionDimensionsPanel.add(heightLabel);
        positionDimensionsPanel.add(heightTextField);
        
        contentPane.add(positionDimensionsPanel);
        
        positionDimensionsTracker = new PositionDimensionsTracker(xTextField, yTextField, widthTextField, heightTextField);
    }

    private void createGroupUngroup(Container contentPane) {
        JPanel groupUngroupPanel = new JPanel(new GridLayout(1, 3));
        TitledBorder groupUngroupBorder = BorderFactory
                        .createTitledBorder("Group/Ungroup");
        groupUngroupPanel.setBorder(groupUngroupBorder);

        String iconPath;
        ImageIcon icon;
        JButton button;

        iconPath = IMAGES + "group/group.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Group", icon);
        button.addActionListener(
                        new CommandActionListener(new GroupCommand("GROUP")));
        groupUngroupPanel.add(button);

        iconPath = IMAGES + "group/ungroup.gif";
        Iconkit.instance().registerImage(iconPath);
        icon = new ImageIcon(Iconkit.instance().getImage(iconPath));
        button = new JButton("Ungroup", icon);
        button.addActionListener(new CommandActionListener(
                        new UngroupCommand("UNGROUP")));
        groupUngroupPanel.add(button);

        contentPane.add(groupUngroupPanel);
    }

    public static void openLayoutWindow() {
        if (LayoutWindow.layoutWindow == null) {
            LayoutWindow.layoutWindow = new LayoutWindow();
            LayoutWindow.layoutWindow.init();
            LayoutWindow.layoutWindow.pack();
        }
        if (LayoutWindow.layoutWindow.isVisible()) {
            LayoutWindow.layoutWindow.toFront();
        } else {
            LayoutWindow.layoutWindow.setVisible(true);
        }
        DrawPlugin.getCurrent().getMenuManager().getWindowsMenu().addFrame(
                        DrawPlugin.WINDOWS_CATEGORY_TOOLS, LayoutWindow.layoutWindow);
    }

    public static void disposeWindow() {
        if (LayoutWindow.layoutWindow != null) {
            DrawPlugin.getCurrent().getMenuManager().getWindowsMenu()
                            .removeFrame(LayoutWindow.layoutWindow);
            LayoutWindow.layoutWindow.dispose();
        }
    }
    
    public static PositionDimensionsTracker getPositionDimensionsTracker() {
        return positionDimensionsTracker;
    }
}
