package de.renew.extras.layoutwindow;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.DrawApplication;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingChangeEvent;
import CH.ifa.draw.framework.DrawingChangeListener;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.util.DrawingListener;

import java.awt.KeyboardFocusManager;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.util.Collections;
import java.util.Vector;

import javax.swing.JFormattedTextField;

public class PositionDimensionsTracker implements DrawingListener, DrawingChangeListener {
    
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(PositionDimensionsTracker.class);

    private JFormattedTextField xTextField;
    private JFormattedTextField yTextField;
    private JFormattedTextField widthTextField;
    private JFormattedTextField heightTextField;

    public PositionDimensionsTracker(JFormattedTextField xTextField, JFormattedTextField yTextField, JFormattedTextField widthTextField,
                                     JFormattedTextField heightTextField) {
        this.xTextField = xTextField;
        this.yTextField = yTextField;
        this.widthTextField = widthTextField;
        this.heightTextField = heightTextField;
        
        this.xTextField.addActionListener(e -> updateFigure());
        this.yTextField.addActionListener(e -> updateFigure());
        this.widthTextField.addActionListener(e -> updateFigure());
        this.heightTextField.addActionListener(e -> updateFigure());
        

        
        xTextField.setFocusTraversalKeys(
                        KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, Collections.EMPTY_SET);
        yTextField.setFocusTraversalKeys(
                        KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, Collections.EMPTY_SET);
        widthTextField.setFocusTraversalKeys(
                        KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, Collections.EMPTY_SET);
        heightTextField.setFocusTraversalKeys(
                        KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, Collections.EMPTY_SET);
        
        this.xTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            @Override
            public void keyPressed(java.awt.event.KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_TAB) {
                    updateFigure();
                    yTextField.grabFocus();
                }
            }
        });
        
        this.yTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            @Override
            public void keyPressed(java.awt.event.KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_TAB) {
                    updateFigure();
                    widthTextField.grabFocus();
                }
            }
        });
        
        this.widthTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            @Override
            public void keyPressed(java.awt.event.KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_TAB) {
                    updateFigure();
                    heightTextField.grabFocus();
                }
            }
        });
        
        this.heightTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            @Override
            public void keyPressed(java.awt.event.KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_TAB) {
                    updateFigure();
                    xTextField.grabFocus();
                }
            }
        });
        
        DrawApplication app = DrawPlugin.getGui();
        app.addDrawingListener(this);
        
        Collections.list(app.drawings()).stream().forEach(d -> d.addDrawingChangeListener(this));
    }

    private void updateFigure() {
        Vector<Figure> selection = DrawPlugin.getGui().view().selection();
        if (selection.size() == 1) {
            try {
                int x = Integer.parseInt(xTextField.getText());
                int y = Integer.parseInt(yTextField.getText());
                int width = Integer.parseInt(widthTextField.getText());
                int height = Integer.parseInt(heightTextField.getText());
                Rectangle r = new Rectangle(x,y,width,height);
                Figure f = selection.firstElement();
                f.displayBox(r);
                f.invalidate();
                f.changed();
                DrawPlugin.getGui().view().checkDamage();                
            } catch(NumberFormatException e) {
                logger.warn(e.getMessage());
            }
        }
    }

    @Override
    public void drawingReleased(Drawing drawing) {
        drawing.removeDrawingChangeListener(this);
    }

    @Override
    public void drawingAdded(Drawing drawing) {
        drawing.addDrawingChangeListener(this);
    }

    @Override
    public void drawingInvalidated(DrawingChangeEvent e) {
        Vector<Figure> selection = DrawPlugin.getGui().view().selection();
        if (selection.size() == 1) {
            Figure f = selection.firstElement();
            Rectangle r = f.displayBox();
            xTextField.setText("" + r.x);
            yTextField.setText("" + r.y);
            widthTextField.setText("" + r.width);
            heightTextField.setText("" + r.height);
        } else {
            xTextField.setText("");
            yTextField.setText("");
            widthTextField.setText("");
            heightTextField.setText("");
        }
    }

    @Override
    public void drawingRequestUpdate(DrawingChangeEvent e) {
    }
}
