package de.renew.extras.statistics;


/**
 * Evaluation.java
 *
 * In order to use real time evaluation it is necessary to start with constructor
 *      Evaluation(double time)
 *
 * Created: Fri May  2 12:02:23 2003
 *
 * @author <a href="mailto: ">Heiko Roelke</a>, Lawrence Cabac
 * @version
 */
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;


public class Evaluation {
    private Hashtable<String, ColoredTransitionEvent> transEvents;

    //private Hashtable placeEvents; for future extensions...
    private double startingTime = 0.0;

    public Evaluation() {
        transEvents = new Hashtable<String, ColoredTransitionEvent>();
    }

    //if the starting time differs from zero
    public Evaluation(double time) {
        transEvents = new Hashtable<String, ColoredTransitionEvent>();
        startingTime = time;
    }

    //colored (default)
    public void transInputEvent(String key, Object value, double time) {
        if (transEvents.containsKey(key)) {
            transEvents.get(key).in(value, time);
        } else {
            ColoredTransitionEvent te = new ColoredTransitionEvent(
                            startingTime);
            te.in(value, time);
            transEvents.put(key, te);
        }
    }

    //black token: to be replaced by new object
    public void transInputEvent(String key, double time) {
        transInputEvent(key, new Object(), time);
    }

    //realtime...
    public void transInputEvent(String key) {
        transInputEvent(key,
                        Long.valueOf(System.currentTimeMillis()).doubleValue());
    }

    public void transInputEvent(String key, Object value) {
        transInputEvent(key, value,
                        Long.valueOf(System.currentTimeMillis()).doubleValue());
    }


    //colored (default)
    public void transOutputEvent(String key, Object value, double time) {
        if (transEvents.containsKey(key)) {
            transEvents.get(key).out(value, time);
        } else {
            //this should not happen...
            ColoredTransitionEvent te = new ColoredTransitionEvent(
                            startingTime);
            te.out(value, time);
            //now the Queue is emptier than empty :)
            transEvents.put(key, te);
        }
    }

    //black token
    public void transOutputEvent(String key, double time) {
        if (transEvents.containsKey(key)) {
            transEvents.get(key).out(time);
        } else {
            //this should not happen...
            ColoredTransitionEvent te = new ColoredTransitionEvent(
                            startingTime);
            te.out(time);
            //now the Queue is emptier than empty :)
            transEvents.put(key, te);
        }
    }

    //realtime
    public void transOutputEvent(String key) {
        transOutputEvent(key,
                        Long.valueOf(System.currentTimeMillis()).doubleValue());
    }

    public void transOutputEvent(String key, Object value) {
        transOutputEvent(key, value,
                        Long.valueOf(System.currentTimeMillis()).doubleValue());
    }

    public void simulationStop(double time) {
        Enumeration<String> allTransEvents = transEvents.keys();
        while (allTransEvents.hasMoreElements()) {
            String key = allTransEvents.nextElement();

            System.out.println("\nReporting for system: " + key + "\n");

            ColoredTransitionEvent te = transEvents
                            .get(key);

            //te.report(time);
            String htmlString = te.reportHTML(key, time);
            writeout(key + ".html", htmlString);
            javax.swing.JLabel display = new javax.swing.JLabel(htmlString);
            javax.swing.JPanel displayPanel = new javax.swing.JPanel();
            displayPanel.add(display);
            javax.swing.JFrame displayFrame = new javax.swing.JFrame();
            displayFrame.setContentPane(displayPanel);
            displayFrame.pack();
            displayFrame.setVisible(true);

            //test html dump
            writeout("test entry", ColoredTransitionEvent.SUFFIX_HTML,
                            te.dumpPopulations(ColoredTransitionEvent.HTML));
            //  	    javax.swing.JLabel anzeige = new javax.swing.JLabel(te.dumpPopulations(ColoredTransitionEvent.HTML));
            //  	    javax.swing.JPanel anzP = new javax.swing.JPanel();
            //  	    anzP.add(anzeige);
            //  	    javax.swing.JFrame anz = new javax.swing.JFrame();
            //  	    anz.setContentPane(anzP);
            //  	    anz.pack();
            //  	    anz.setVisible(true);
            if (true) { //fillings
                String tempDat = writeout("test.dat",
                                te
                                                .dumpPopulations());
                String header = "#!/usr/local/bin/gnuplot -persist\n"
                                + "set title \"Population of system: " + key
                                + "\"\n" + "set xlabel \"time\"\n"
                                + "set ylabel \"population\"\n" + "plot \""
                                + tempDat + "\" with steps\n";
                String tempPlt = writeout("test.plt", header);
                try {
                    String[] cmd = { "/bin/sh", "-c", "gnuplot -persist < "
                                    + tempPlt };
                    Runtime.getRuntime().exec(cmd);
                } catch (Exception e) {
                    System.out.println("das war nix...");
                }
            }
            if (true) { //dwells vs fillings
                String tempDat = writeout("test2.dat",
                                te
                                                .dumpDelayVsPopulations());
                String header = "#!/usr/local/bin/gnuplot -persist\n"
                                + "set title \"Delay vs population for: " + key
                                + "\"\n" + "set xlabel \"population\"\n"
                                + "set ylabel \"mean delay\"\n" + "plot \""
                                + tempDat + "\" with boxes\n";
                String tempPlt = writeout("test2.plt", header);
                try {
                    String[] cmd = { "/bin/sh", "-c", "gnuplot -persist < "
                                    + tempPlt };
                    Runtime.getRuntime().exec(cmd);
                } catch (Exception e) {
                    System.out.println("das war nix...");
                }
            }

            System.out.println("End of report--------\n\n\n");

        }
    }

    //realtime
    public void simulationStop() {
        simulationStop(Long.valueOf(System.currentTimeMillis()).doubleValue());
    }

    @Override
    public String toString() {
        return transEvents.toString();
    }

    private String writeout(String dateiPre, String dateiSuf,
                            String zuSchreiben, File dir) {
        File f;
        FileWriter out = null;
        String filename = "";

        try {
            f = File.createTempFile(dateiPre, dateiSuf, dir);
            filename = f.getName();
            out = new FileWriter(f);
            out.write(zuSchreiben, 0, zuSchreiben.length());
            out.flush();
        } catch (IOException e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
            }
        }

        System.out.println("File: " + filename + "\n\n");


        return filename;
    }

    private String writeout(String datei, String zuSchreiben, File dir) {
        return writeout(datei.substring(0, datei.lastIndexOf('.')),
                        datei.substring(datei.lastIndexOf('.'), datei.length()),
                        zuSchreiben, dir);
    }

    private String writeout(String dateiPre, String dateiSuf,
                            String zuSchreiben) {
        return writeout(dateiPre, dateiSuf, zuSchreiben,
                        new File(System.getProperty("user.dir")));
    }

    private String writeout(String datei, String zuSchreiben) {
        return writeout(datei, zuSchreiben,
                        new File(System.getProperty("user.dir")));
    }
} // Evaluation
