package de.renew.plugin.load;

import java.lang.module.ModuleReference;
import java.util.Collection;
import java.util.List;

public class ModuleLayerCreationException extends RuntimeException {
    public ModuleLayerCreationException(String moduleToLoad, Collection<Module> loadedModules) {
        super("While loading the module " + moduleToLoad + ", " +
                "the Java Module System attempted to include one or more different modules in the same layer." +
                "This is usually the result of an incorrect loading order of plugins. " +
                "The following modules were included in this layer: " +
                loadedModules.stream().map(Module::getName).reduce("", String::concat) + " " +
                "Please make sure that the additional modules are loaded before " + moduleToLoad + ".");
    }

    public ModuleLayerCreationException(ModuleReference moduleToLoad,
                                        List<String> requiredLayers,
                                        List<ModuleLayer> foundLayers) {
        super("Couldn't find all of the required parent layers for module " + moduleToLoad.descriptor().name() + ". " +
                "Required parent layers: " + requiredLayers.toString() + ". " +
                "Found layers: " + foundLayers.toString() + ". " +
                "This is usually the result of an incorrect loading order of plugins. " +
                "Please make sure that all required modules are loaded before " + moduleToLoad.descriptor().name() + ".");
    }
}
