package de.renew.logging.gui.test;

import de.renew.application.SimulatorPlugin;
import de.renew.application.StartSimulationCommand;

import de.renew.engine.common.SimulatorEventLogger;
import de.renew.engine.events.NetInstantiation;
import de.renew.engine.events.TraceEvent;

import de.renew.logging.gui.*;

import de.renew.plugin.PluginProperties;

import org.apache.log4j.Logger;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Tests the correct creation of {@link StepTrace}s as well as their retrieval from
 * a {@link StepTraceRepository}.
 * This class is currently limited to test the correct messages for the simulation
 * instantiation only. It can be expanded to include an entire simulation of a
 * given net. See comment in {@link #testStepTraceMessageGeneration()}
 */
public class StepTraceTest {

    private SimulatorPlugin simulatorPlugin;

    @Before
    public void startSimulation() {
        // Register the GuiAppender for the current Logger
        Logger.getLogger(SimulatorEventLogger.SIM_LOG_PREFIX).addAppender(new GuiAppender());

        simulatorPlugin = new SimulatorPlugin(PluginProperties.getUserProperties());
        StartSimulationCommand startSimulationCommand = new StartSimulationCommand(simulatorPlugin);

        String sampleNetPath = getClass()
                .getResource("/de/renew/logging/gui/test/simplenet1.sns")
                .getPath();

        String[] sscArgs = new String[3];
        sscArgs[0] = sampleNetPath; // path to compiled net (sns) file
        sscArgs[1] = "simplenet1"; // name of the main net to simulate
        sscArgs[2] = "-i"; // only initialize simulation, don't run immediately

        startSimulationCommand.execute(sscArgs, System.out);
        assertTrue(simulatorPlugin.isSimulationActive());
    }

    @Test
    public void testStepTraceMessageGeneration() {

        String[] loggerNames = LoggingController.findSimulationLoggerNames();
        String loggerToUse = loggerNames[0];

        assertEquals("simulation", loggerToUse);

        StepTraceRepository currentRepository = MainRepositoryManager
                .getInstance()
                .getCurrentRepository(loggerToUse);

        assertTrue("No step traces found!",
                currentRepository.getAllStepTraces().length > 0);

        StepTrace stepTraceOne = currentRepository.getAllStepTraces()[0];
        StepTableModel stepTableModelOne = new StepTableModel(stepTraceOne, new RepositoryChangeBuffer());
        NetInstantiation eventOne = (NetInstantiation) stepTableModelOne.getValueAt(0, 0);
        assertEquals("New net instance simplenet1[0] created.", eventOne.toString());
        TraceEvent eventTwo = (TraceEvent) stepTableModelOne.getValueAt(1, 0);
        assertEquals("Initializing [] into simplenet1[0].p1", eventTwo.toString());

        // TODO: The following part has issues with concurrency in the simulator
        // We need a way to find out if the simulator has finished a simulation step
        // (or alternatively wait for the entire simulation to finish).
        // After that we can evaluate the events of all StepTrace objects inside
        // currentRepository.getAllStepTraces() like sketched below

        //simulatorPlugin.getCurrentEnvironment().getSimulator().step();
        ////Thread.sleep(500); // Wait for simulator thread to complete step
        //simulatorPlugin.getCurrentEnvironment().getSimulator().step();
        ////Thread.sleep(500); // Wait for simulator thread to complete step
        //simulatorPlugin.getCurrentEnvironment().getSimulator().step();

        //StepTrace secondStepTrace3 = currentRepository.getAllStepTraces()[1];
        //StepTableModel stepTableModel3 = new StepTableModel(secondStepTrace3, new RepositoryChangeBuffer());
        //TraceEvent stepMessage4 = (TraceEvent) stepTableModel3.getValueAt(0, 0);
        //assertEquals("-------- Synchronously --------", stepMessage4.toString());
        //Removing secondStepMessagetrace4 = (Removing) stepTableModel3.getValueAt(1, 0);
        //assertEquals("Removing [] in simplenet1[0].p1", secondStepMessagetrace4.toString());


        //StepTableModel stepTableModel2 = new StepTableModel(secondStepTrace3, new RepositoryChangeBuffer());
        //Firing stepMessage2 = (Firing) stepTableModel2.getValueAt(3, 0);
        //assertEquals("Firing simplenet1[0].t1", stepMessage2.toString());

        //Putting secondStepMessagetrace = (Putting) stepTableModel2.getValueAt(1, 0);
        //assertEquals("Putting [] into simplenet1[0].p2", secondStepMessagetrace.toString());

    }
}
