package de.renew.minimap.command;

import CH.ifa.draw.DrawPlugin;
import CH.ifa.draw.application.MenuManager;
import CH.ifa.draw.util.Command;

import bibliothek.gui.dock.common.DefaultSingleCDockable;

import de.renew.minimap.component.MiniMapPanel;
import de.renew.plugin.command.CLCommand;
import de.renew.windowmanagement.Workbench;
import de.renew.windowmanagement.WorkbenchImpl;

import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JMenuItem;
import javax.swing.AbstractAction;


//import javax.swing.JButton;


//import java.awt.BorderLayout;


/**
 * This command adds a {@link bibliothek.gui.dock.common.DefaultSingleCDockable} with a MiniMap.
 *
 * @author Christian Roeder
 */
public class ShowMiniMapCommand extends Command implements CLCommand {
    private static final org.apache.log4j.Logger logger = org.apache.log4j.Logger
                    .getLogger(ShowMiniMapCommand.class);

    /**
     * The name of this command that is displayed throughout the project
     */
    public static final String COMMAND_NAME = "Show MiniMap...";

    //Frame containing the MiniMapFrame.
    private DefaultSingleCDockable frame;

    //a menu item for the windows menu.
    private JMenuItem menuItem;
    private Workbench workbench;

    /**
     * Instantiates a new show mini map command.
     *
     * @param workbench workbench to add the window to
     */
    public ShowMiniMapCommand(Workbench workbench) {
        super(COMMAND_NAME);
        this.workbench = workbench;
    }

    /**
     * {@link CH.ifa.draw.util.Command#execute()}
     */
    @Override
    public void execute() {
        showMiniMap();
    }

    /**
     * {@link de.renew.plugin.command.CLCommand#execute(java.lang.String[], java.io.PrintStream)}
     *
     * @param args      The command line that was typed by the user.
     * @param response  The <code>PrintStream</code> for user feedback.
     */
    @Override
    public void execute(final String[] args, final PrintStream response) {
        try {
            EventQueue.invokeAndWait(new Runnable() {
                @Override
                public void run() {
                    response.println("Show MiniMap.");
                    execute();
                }
            });
        } catch (InterruptedException e) {
            // unexpected, but harmless.
        } catch (InvocationTargetException e) {
            response.println("Exception while showing MiniMap: "
                            + e.getTargetException());
            logger.error(e.getTargetException().toString(), e);
        }
    }

    /**
     * Gets the arguments.
     *
     * @return the arguments
     * {@link de.renew.plugin.command.CLCommand#getArguments()}
     */
    @Override
    public String getArguments() {
        return null;
    }

    /**
     * {@link de.renew.plugin.command.CLCommand#getDescription()}
     *
     * @return the description of this command
     */
    @Override
    public String getDescription() {
        return "Opens a window where commands can be executed.";
    }

    /**
     * Adds an menu item for the MiniMap frame to the windows menu.
     */
    private void addMenuItemToWindowsMenu() {
        if (menuItem != null) {
            return;
        }
        menuItem = new JMenuItem(new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent e) {
                bringToFront();
            }
        });
        menuItem.setText(COMMAND_NAME);
        MenuManager.getInstance().getWindowsMenu().add(menuItem);
    }

    /**
     * Bring the MiniMap frame to the front.
     */
    private void bringToFront() {
        if (frame != null) {
            frame.setVisible(true);
            frame.toFront();
        }
    }

    /**
     * Close MiniMap frame.
     */
    private void closeMiniMapFrame() {
        removeMenuItemFromWindowsMenu();
        destroyMiniMapFrame();
    }

    /**
     * Create the MiniMap frame.
     */
    private void createMiniMapFrame() {
        DefaultSingleCDockable miniMapFrame = new DefaultSingleCDockable(
                        "Mini Map");
        miniMapFrame.setTitleText("Mini Map");
        miniMapFrame.setCloseable(true);
        MiniMapPanel miniMapPanel = new MiniMapPanel();
        miniMapPanel.setPreferredSize(new Dimension(320, 640));
        miniMapPanel.setVisible(true);
        miniMapFrame.add(miniMapPanel);

        workbench.addViewWindow(miniMapFrame, WorkbenchImpl.UPPER_RIGHT_GROUP);
        workbench.registerFocusListener(miniMapPanel);
        frame = miniMapFrame;


    }

    /**
     * Dispose of the MiniMapFrame.
     */
    private void destroyMiniMapFrame() {
        frame.getControl().removeDockable(frame);
        frame = null;
    }

    /**
     * Removes the menu item from the windows menu.
     */
    private void removeMenuItemFromWindowsMenu() {
        DrawPlugin.getCurrent().getMenuManager().getWindowsMenu()
                        .remove(menuItem);
        menuItem = null;
    }

    /**
     * Show MiniMap.
     */
    private void showMiniMap() {
        if (frame != null) {
            frame.setVisible(true);
        } else {
            createMiniMapFrame();
            addMenuItemToWindowsMenu();
        }
    }
}