package de.renew.ontology;

import de.renew.ontology.generators.JavaNoAgentOntologyGenerator;
import de.renew.ontology.generators.JavaOntologyGenerator;
import de.renew.ontology.generators.JavaScriptAmdOntologyGenerator;
import de.renew.ontology.generators.JavaScriptOntologyGenerator;
import de.renew.ontology.providers.FSOntologyProvider;
import de.renew.plugin.PluginAdapter;
import de.renew.plugin.PluginException;
import de.renew.plugin.PluginProperties;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;


/**
 * @author Konstantin Simon Maria Moellers
 * @author Christian Bracker (1bracker)
 * @version 2015-11-29
 */
public class OntologyPlugin extends PluginAdapter {
    private static final Map<String, OntologyProvider> ONTOLOGY_PROVIDERS = new HashMap<String, OntologyProvider>();
    private static final Map<String, OntologyGenerator> ONTOLOGY_GENERATORS = new HashMap<String, OntologyGenerator>();

    static {
        registerClasses();
    }

    public OntologyPlugin(URL location) throws PluginException {
        super(location);
    }

    public OntologyPlugin(PluginProperties props) {
        super(props);
    }

    /**
     * Finds an ontology provider by its name.
     */
    public static OntologyProvider findProvider(String name) {
        return ONTOLOGY_PROVIDERS.get(name);
    }

    /**
     * Finds an ontology generator by its name.
     */
    public static OntologyGenerator findGenerator(String name) {
        return ONTOLOGY_GENERATORS.get(name);
    }

    private static void registerClasses() {
        registerProvider(new FSOntologyProvider());

        registerGenerator(new JavaOntologyGenerator());
        registerGenerator(new JavaNoAgentOntologyGenerator());
        registerGenerator(new JavaScriptOntologyGenerator());
        registerGenerator(new JavaScriptAmdOntologyGenerator());
    }

    private static void registerProvider(OntologyProvider provider) {
        ONTOLOGY_PROVIDERS.put(provider.getProviderName(), provider);
    }

    private static void registerGenerator(OntologyGenerator generator) {
        ONTOLOGY_GENERATORS.put(generator.getGeneratorName(), generator);
    }
}