package de.renew.ontology;

import de.renew.ontology.entities.Ontology;
import de.renew.ontology.exception.OntologyProviderException;

import java.io.File;
import java.util.Collection;


/**
 * @author Konstantin Simon Maria Moellers
 * @version 2015-01-14
 */
public interface OntologyProvider {

    /**
     * Finds ontology models by given net names.
     *
     * @return ontology models representing the nets
     * @throws de.renew.ontology.exception.OntologyProviderException
     */
    public Collection<Ontology> provide() throws OntologyProviderException;

    /**
     * Returns a name string representing this provider.
     *
     * @return string representing the name.
     */
    public String getProviderName();

    /**
     * Sets the source files of the provider.
     *
     * @param sourceFiles files used for provisioning
     */
    public void setSourceFiles(Collection<File> sourceFiles);

    /**
     * Gets the source files of the provider.
     *
     * @return files used for provisioning
     */
    public Collection<File> getSourceFiles();

    /**
     * Sets the net path. 
     * 
     * @param netPath path to search for nets 
     */
    public void setNetPath(Collection<String> netPath);
}