package de.renew.ontology.ant;

import de.renew.ontology.OntologyGenerator;
import de.renew.ontology.OntologyPlugin;
import de.renew.ontology.exception.OntologyGeneratorException;

import java.io.File;


/**
 * @author Konstantin Simon Maria Moellers
 * @version 2015-02-24
 */
public class GeneratorReference {
    private String name;
    private String className;
    private File destinationDir;

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public File getDestinationDir() {
        return destinationDir;
    }

    public void setDestinationDir(File destinationDir) {
        this.destinationDir = destinationDir;
    }

    /**
     * Creates an ontology generator based on reference information.
     * @throws OntologyGeneratorException
     */
    public OntologyGenerator createGenerator()
                    throws OntologyGeneratorException {
        OntologyGenerator generator = null;
        if (name != null) {
            generator = OntologyPlugin.findGenerator(name);
            if (generator == null) {
                throw new OntologyGeneratorException(
                                "Could not find generator for name: " + name);
            }
        } else if (className != null) {
            try {
                Class<?> clazz = Class.forName(className);
                Object instance = clazz.newInstance();
                if (instance instanceof OntologyGenerator) {
                    generator = (OntologyGenerator) instance;
                } else {
                    throw new OntologyGeneratorException(
                                    "Configured class is not an ontology generator: "
                                                    + className);
                }
            } catch (ClassNotFoundException e) {
                throw new OntologyGeneratorException(
                                "Could not find generator for class: "
                                                + className,
                                e);
            } catch (InstantiationException e) {
                throw new OntologyGeneratorException(
                                "Could instantiate generator for class: "
                                                + className,
                                e);
            } catch (IllegalAccessException e) {
                throw new OntologyGeneratorException(
                                "Could instantiate generator for class: "
                                                + className,
                                e);
            }
        } else {
            throw new OntologyGeneratorException(
                            "Generator not properly configured. Neither name nor className is set.");
        }

        generator.setDestinationDirectory(destinationDir);

        return generator;
    }
}