package de.renew.ontology.ant;

import de.renew.ontology.OntologyConverter;
import de.renew.ontology.exception.OntologyGeneratorException;
import de.renew.ontology.exception.OntologyProviderException;

import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;


/**
 * This is the Ant task <code>&lt;ontology/&gt;</code>.
 *
 * @author Konstantin Simon Maria Moellers
 */
public class OntologyTask extends Task {

    /**
     * Providers being referenced in the task.
     */
    private Vector<ProviderReference> providers = new Vector<ProviderReference>();

    /**
     * Generators being referenced in the task.
     */
    private Vector<GeneratorReference> generators = new Vector<GeneratorReference>();

    public ProviderReference createProvider() {
        ProviderReference providerReference = new ProviderReference(this);
        this.providers.add(providerReference);

        return providerReference;
    }

    public void addGenerator(GeneratorReference generator) {
        this.generators.add(generator);
    }

    @Override
    public void execute() throws BuildException {
        OntologyConverter converter = new OntologyConverter();
        try {
            addProvidersToConverter(converter);
            addGeneratorsToConverter(converter);

            converter.convert();
        } catch (OntologyProviderException e) {
            e.printStackTrace();
            throw new BuildException("Could not provide ontology model", e);
        } catch (OntologyGeneratorException e) {
            e.printStackTrace();
            throw new BuildException("Could not generate ontology model", e);
        }
    }

    /**
     * Fills an ontology converter with providers from the Ant task.
     *
     * @param converter a converter to fill with providers
     * @throws OntologyProviderException
     */
    private void addProvidersToConverter(OntologyConverter converter)
                    throws OntologyProviderException {
        for (ProviderReference providerReference : providers) {
            converter.addProvider(providerReference.createProvider());
        }
    }

    /**
     * Fills an ontology converter with generators from the Ant task.
     *
     * @param converter a converter to fill with generators
     * @throws OntologyGeneratorException
     */
    private void addGeneratorsToConverter(OntologyConverter converter)
                    throws OntologyGeneratorException {
        for (GeneratorReference generatorReference : generators) {
            converter.addGenerator(generatorReference.createGenerator());
        }
    }
}