package de.renew.ontology.ant;

import de.renew.ontology.OntologyPlugin;
import de.renew.ontology.OntologyProvider;
import de.renew.ontology.exception.OntologyProviderException;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;

import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;


/**
 * @author Konstantin Simon Maria Moellers
 * @version 2015-02-24
 */
public class ProviderReference {
    private OntologyTask parent;
    private String name;
    private String className;
    private File sourceFile;
    private Vector<FileSet> filesets = new Vector<FileSet>();
    private Path netPath;

    public ProviderReference(OntologyTask parent) {
        this.parent = parent;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public File getSrc() {
        return sourceFile;
    }

    public void setSrc(File sourceFile) {
        this.sourceFile = sourceFile;
    }

    public void addConfiguredFileset(FileSet fileset) {
        this.filesets.add(fileset);
    }


    /**
     * Sets the reference to the ontology net path.
     *
     * @param netPathRef the "net path"-property name.
     */
    public void setNetPathRef(Reference netPathRef) {
        this.netPath = (Path) netPathRef
                        .getReferencedObject(parent.getProject());
    }

    /**
     * Creates an ontology provider based on reference information.
     */
    public OntologyProvider createProvider() throws OntologyProviderException {
        OntologyProvider provider = null;
        if (name != null) {
            provider = OntologyPlugin.findProvider(name);
            if (provider == null) {
                throw new OntologyProviderException(
                                "Could not find provider for name: " + name);
            }
        } else if (className != null) {
            try {
                Class<?> clazz = Class.forName(className);
                if (OntologyProvider.class.isAssignableFrom(clazz)) {
                    provider = (OntologyProvider) clazz.getDeclaredConstructor()
                                    .newInstance();
                } else {
                    throw new OntologyProviderException(
                                    "Configured class is not an ontology provider: "
                                                    + className);
                }

            } catch (ClassNotFoundException | InstantiationException
                            | IllegalAccessException | IllegalArgumentException
                            | InvocationTargetException | NoSuchMethodException
                            | SecurityException e) {
                throw new OntologyProviderException(
                                "Could instantiate provider for class: "
                                                + className,
                                e);
            }
        } else {
            throw new OntologyProviderException(
                            "Provider not properly configured. Neither name nor className is set.");
        }


        if (sourceFile != null && this.filesets.size() > 0) {
            throw new OntologyProviderException(
                            "Source file and FileSet specified. You can't use both.");
        } else if (sourceFile != null && !sourceFile.exists()) {
            throw new OntologyProviderException(
                            "Source file does not exist: " + sourceFile);
        } else if (sourceFile == null && this.filesets.size() == 0) {
            throw new OntologyProviderException("No source files specified");
        } else if (sourceFile != null && sourceFile.exists()) {
            provider.setSourceFiles(Arrays.asList(sourceFile));
        } else {
            ArrayList<File> sourceFiles = new ArrayList<File>();
            for (FileSet fs : this.filesets) {
                DirectoryScanner ds = fs.getDirectoryScanner();
                File dir = ds.getBasedir();
                for (String net : ds.getIncludedFiles()) {
                    File file = new File(dir + File.separator + net);
                    sourceFiles.add(file);
                }
            }
            provider.setSourceFiles(sourceFiles);
        }

        provider.setNetPath(Arrays.asList(this.netPath.list()));

        return provider;
    }

}