package de.renew.ant;

import java.io.File;
import java.io.IOException;

import org.antlr.v4.Tool;
import org.antlr.v4.tool.ErrorType;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Mkdir;
import org.apache.tools.ant.types.FileSet;


public class Antlr4Task extends Task {
    protected File destdir;
    protected FileSet fileset;
    protected File srcbase;
    protected String singleTaskName;

    public void setDestdir(File destdir) {
        this.destdir = destdir;
    }

    public void addConfiguredFileset(FileSet fileset) {
        this.fileset = fileset;
        this.srcbase = fileset.getDir(getProject());
    }

    public void setSrcbase(File srcbase) {
        this.srcbase = srcbase;
    }

    public void execute() throws BuildException {
        DirectoryScanner ds = fileset.getDirectoryScanner(getProject());
        String[] antlrGrammarFiles = ds.getIncludedFiles();
        File basedir = ds.getBasedir();
        for (int i = 0; i < antlrGrammarFiles.length; i++) {
            File f = new File(basedir.getAbsolutePath() + File.separator
                            + antlrGrammarFiles[i]);
            if (!f.isFile()) {
                throw new BuildException("antlr4 grammar file " + f
                                + " does not exist!");
            }
            mkdirAndExecuteAntlr(f);
        }
    }

    protected void mkdirAndExecuteAntlr(File antlrGrammarFile)
                    throws BuildException {
        File directory = getGenerationDirectory(antlrGrammarFile);

        // Create dir if not exist
        if (!directory.isDirectory()) {
            Mkdir mkdir = (Mkdir) getProject().createTask("mkdir");
            mkdir.setDir(directory);
            mkdir.init();
            mkdir.execute();
        }
        executeAntlr(antlrGrammarFile, directory);
    }

    protected void executeAntlr(File antlrGrammarFile, File directory)
                    throws BuildException {
        // get the package quick and dirty
        String targetPackage = antlrGrammarFile.getParent().toString()
                        .substring(this.srcbase.toString().length() + 1)
                        .replace("/", ".");
        String[] args = new String[] { antlrGrammarFile
                .toString(), "-o", directory
                .toString(), "-package", targetPackage };
        Tool antlr = new Tool(args);
        try {
            antlr.processGrammarsOnCommandLine();
        }
        finally {
            if ( antlr.log ) {
                try {
                    String logname = antlr.logMgr.save();
                    System.out.println("wrote "+logname);
                }
                catch (IOException ioe) {
                    antlr.errMgr.toolError(ErrorType.INTERNAL_ERROR, ioe);
                }
            }
        }
        if (antlr.errMgr.getNumErrors() > 0) {
            throw new BuildException();
        }
    }

    protected File getGenerationDirectory(File antlrGrammarFile) {
        String targetDir = antlrGrammarFile.getParent();
        String packageDir = targetDir
                        .substring(srcbase.getAbsolutePath().length());
        File directory = new File(destdir.getAbsolutePath() + File.separator
                        + packageDir);
        return directory;
    }

    public static void main(String[] args) {
        var srcbase = "/Users/doodles/AOSE/workspace/Renew/EVE/src";
        var file = "/Users/doodles/AOSE/workspace/Renew/EVE/src/de/renew/eve/parser/grammars/Expr.g4";
        var targetPackage = new File(file.substring(srcbase.length() + 1))
                        .getParent().toString().replace("/", ".");
        System.out.println(targetPackage);

    }

}
