package de.renew.splashscreen;

import de.renew.plugin.PluginAdapter;
import de.renew.plugin.PluginException;
import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;
import de.renew.plugin.load.PluginLoaderComposition;

import java.net.URL;


/**
 * The main class of the {@code SplashscreenPlugin}.
 *
 * Provides a singleton instance of this plugin
 *
 * @author Dominic Dibbern
 * @date Jan 23, 2012
 * @version 0.1
 */
public class SplashscreenPlugin extends PluginAdapter {
    private static final String KEY_SPLASHSCREEN_ENABLED = "de.renew.splashscreen.enabled";
    private static SplashscreenPlugin _instance;

    /**
     * Construct a new instance of the {@code SplashscreenPlugin}.
     * This constructor is called via reflection from
     * {@link de.renew.plugin.load.AbstractPluginLoader}
     *
     * @param propertyURL URL to the properties file
     * @throws PluginException if the instantiation failed.
     */
    public SplashscreenPlugin(URL propertyURL) throws PluginException {
        super(propertyURL);
        makeInstance();
        showScreen();
    }

    /**
     * Construct a new instance of the {@code SplashscreenPlugin}.
     * This constructor is called via reflection from
     * {@link de.renew.plugin.load.AbstractPluginLoader}
     *
     * @param props properties for this plugin
     */
    public SplashscreenPlugin(PluginProperties props) {
        super(props);
        makeInstance();
        showScreen();
    }

    private void makeInstance() {
        if (_instance == null) {
            _instance = this;
        }
    }

    private void showScreen() {
        if (getProperties().getBoolProperty(KEY_SPLASHSCREEN_ENABLED)) {
            RenewSplashScreen renewSplashScreen = RenewSplashScreen
                            .getInstance();
            if (renewSplashScreen != null) {
                renewSplashScreen.showSplashScreen(
                                PluginManager.getLoaderLocation());
                PluginLoaderComposition.setProgressBar(
                                ExtendedProgressBar.getInstance());
            }
        } else {
            logger.debug("Splash screen disabled by user choice.");
        }
    }

    /**
     * Return a singleton instance of this plugin
     * @return the current instance
     */
    public static SplashscreenPlugin getInstance() {
        return _instance;
    }
}